/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_log_fstream_writer_hpp_included
#define cool_log_fstream_writer_hpp_included

/**
 * \file fstream_writer.hpp
 * \author Sven Suursoho
 *
 * \brief Class(es): cool::log_fstream_writer
 *
 * In module \ref cool_log.
 */

#include <cool/cool.hpp>
#include <cool/log/stream_writer.hpp>
#include fstream_header


namespace cool {
/** \addtogroup cool_log */
/** \{ */


/**
 * \brief
 */
template <typename format>
class COOL_API log_fstream_writer: public log_stream_writer<format>
{
  public:

      log_fstream_writer (const char *filename)
          : log_stream_writer<format>(fstream_)
          , filename_(filename) {
          open();
      }


      virtual ~log_fstream_writer () {
          close();
      }


  private:

      bool open () {
          assert(filename_ != 0);
          fstream_.open(filename_, STD_ ios_base::out | STD_ ios_base::app);
          return fstream_.is_open();
      }


      bool reopen () {
          close();
          return open();
      }


      void close () {
          fstream_.close();
      }


      const char *filename_;
      STD_ ofstream fstream_;
};


/** \} */
}   // namespace cool


#endif  /* cool_log_fstream_writer_hpp_included */
