/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_log_syslog_writer_hpp_included
#define cool_log_syslog_writer_hpp_included

/**
 * \file syslog_writer.hpp
 * \author Sven Suursoho
 *
 * \brief Class(es): cool::syslog_writer
 *
 * In module \ref cool_log.
 */

#include <cool/cool.hpp>
#include <cool/log/log.hpp>
#include <cool/log/writer.hpp>
#include <cool/log/plain_format.hpp>
#include <syslog.h>


namespace cool {
/** \addtogroup cool_log */
/** \{ */


/**
 * \brief
 */
template <typename format>
class COOL_API log_syslog_writer: public log_writer<format>
{
  public:

      enum facility_type {
          auth = LOG_AUTH
          , authpriv = LOG_AUTHPRIV
          , cron = LOG_CRON
          , daemon = LOG_DAEMON
          , ftp = LOG_FTP
          , kern = LOG_KERN
          , local0 = LOG_LOCAL0
          , local1 = LOG_LOCAL1
          , local2 = LOG_LOCAL2
          , local3 = LOG_LOCAL3
          , local4 = LOG_LOCAL4
          , local5 = LOG_LOCAL5
          , local6 = LOG_LOCAL6
          , local7 = LOG_LOCAL7
          , lpr = LOG_LPR
          , mail = LOG_MAIL
          , news = LOG_NEWS
          , syslog = LOG_SYSLOG
          , user = LOG_USER
          , uucp = LOG_UUCP
      };


      enum {
          console = LOG_CONS
          , no_delay = LOG_NDELAY
          , open_delay = LOG_ODELAY
          , no_wait = LOG_NOWAIT
          , perror = LOG_PERROR
          , pid = LOG_PID
      };


      log_syslog_writer (const char *ident,
              int o = open_delay, facility_type f = user) {
          ::openlog(ident, o, f);
      }


  private:

      int level_to_int (log::level_type level) {
          switch (level) {
              case log::level_emerg:
                  return LOG_EMERG;

              case log::level_alert:
                  return LOG_ALERT;

              case log::level_crit:
                  return LOG_CRIT;

              case log::level_err:
                  return LOG_ERR;

              case log::level_warn:
                  return LOG_WARNING;

              case log::level_notice:
                  return LOG_NOTICE;

              case log::level_info:
                  return LOG_INFO;

              case log::level_debug:
              default:
                  return LOG_DEBUG;
          }
      }


      void log (log::level_type level, const char *msg) {
          ::syslog(level_to_int(level), msg);
      }
};


/** \} */
}   // namespace cool


#endif  /* cool_log_syslog_writer_hpp_included */
