/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_log_writer_hpp_included
#define cool_log_writer_hpp_included

/**
 * \file writer.hpp
 * \author Sven Suursoho
 *
 * \brief Class(es): cool::log_writer
 *
 * In module \ref cool_log.
 */

#include <cool/cool.hpp>
#include <cool/log/log.hpp>
#include <cstdarg>


namespace cool {
/** \addtogroup cool_log */
/** \{ */


/**
 * \brief
 */
template <typename format>
class COOL_API log_writer: public log::basic_writer
{
  public:

      log_writer (): level_(log::level_unset) {}


      log_writer &level (log::level_type l) {
          level_ = l;
          return *this;
      }


      log::level_type level () const {
          return level_;
      }


      bool enabled (log::level_type l) const {
          return l <= level_;
      }


      log_writer &writeva (log::level_type level, const char *fmt, va_list va) {
          assert(fmt != 0);
          if (enabled(level)) {
              char line[1024];
              vsnprintf(line, sizeof(line), fmt, va);
              operator()(log::event_type(level, line));
          }
          return *this;
      }


      log_writer &write (log::level_type level, const char *fmt, ...);
      log_writer &emerg (const char *fmt, ...);
      log_writer &alert (const char *fmt, ...);
      log_writer &crit (const char *fmt, ...);
      log_writer &err (const char *fmt, ...);
      log_writer &warn (const char *fmt, ...);
      log_writer &notice (const char *fmt, ...);
      log_writer &info (const char *fmt, ...);
      log_writer &debug (const char *fmt, ...);


  private:

      void operator() (const log::event_type &ev) {
          if (enabled(ev.level)) {
              format f;
              log(ev.level, f(ev));
          }
      }

      virtual void log (log::level_type level, const char *msg) = 0;

      log::level_type level_;
};


template <typename format>
log_writer<format> &
log_writer<format>::write (log::level_type level, const char *fmt, ...)
{
  assert(fmt != 0);
  va_list va;
  va_start(va, fmt);
  writeva(level, fmt, va);
  va_end(va);
  return *this;
}


#define DEFINE_LOG_WRITER_METHOD(level) \
    template <typename format> \
    log_writer<format> & \
    log_writer<format>::level (const char *fmt, ...) \
    { \
      assert(fmt != 0); \
      va_list va; \
      va_start(va, fmt); \
      writeva(log::level_##level, fmt, va); \
      va_end(va); \
      return *this; \
    }

DEFINE_LOG_WRITER_METHOD(emerg)
DEFINE_LOG_WRITER_METHOD(alert)
DEFINE_LOG_WRITER_METHOD(crit)
DEFINE_LOG_WRITER_METHOD(err)
DEFINE_LOG_WRITER_METHOD(warn)
DEFINE_LOG_WRITER_METHOD(notice)
DEFINE_LOG_WRITER_METHOD(info)
DEFINE_LOG_WRITER_METHOD(debug)

#undef DEFINE_LOG_WRITER_METHOD


/** \} */
}   // namespace cool


#endif  /* cool_log_writer_hpp_included */
