/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_net_hpp_included
#define cool_net_hpp_included

/**
 * \defgroup cool_net Simple networking
 * \ingroup cool
 * \{
 * Base level networking objects (addresses, sockets etc)
 *
 * Header(s):
 * basic networking
 *  - \ref net_exception.hpp
 *  - \ref inet_address.hpp
 *  - \ref socket_base.hpp
 *  - \ref socket.hpp
 *  - \ref client_socket.hpp
 *  - \ref server_socket.hpp
 *  - \ref inet_socket.hpp
 *  - \ref local_socket.hpp
 *  - \ref poller.hpp
 *  - \ref socket_pair.hpp
 *  - \ref url.hpp
 *
 * iostream-based networking
 *  - \ref socketbuf.hpp
 *  - \ref socketstream.hpp
 *  - \ref inetbuf.hpp
 *  - \ref inetstream.hpp
 *  - \ref localbuf.hpp
 *  - \ref localstream.hpp
 * \}
 */

// basic networking
#include <cool/net/client_socket.hpp>
#include <cool/net/inet_address.hpp>
#include <cool/net/inet_socket.hpp>
#include <cool/net/local_socket.hpp>
#include <cool/net/exception.hpp>
#include <cool/net/poller.hpp>
#include <cool/net/server_socket.hpp>
#include <cool/net/socket.hpp>
#include <cool/net/socket_base.hpp>
#include <cool/net/socket_pair.hpp>
#include <cool/net/url.hpp>

// iostream-based networking
#include <cool/net/inetbuf.hpp>
#include <cool/net/inetstream.hpp>
#include <cool/net/localbuf.hpp>
#include <cool/net/localstream.hpp>
#include <cool/net/socketbuf.hpp>
#include <cool/net/socketstream.hpp>


#endif  /* cool_net_hpp_included */
