/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_net_bits_inet_address_hpp_included
#define cool_net_bits_inet_address_hpp_included

#include <cool/net/socket_base.hpp>

#ifdef HAVE_NETINET_IN_H
#  include <netinet/in.h>
#endif
#ifdef HAVE_ARPA_INET_H
#  include <arpa/inet.h>
#endif
#ifdef HAVE_NETDB_H
#  include <netdb.h>
#endif

#ifndef INADDR_NONE
#  define INADDR_NONE   ((in_addr_t) -1)
#endif


namespace cool {
namespace bits {
namespace net {


// Well-known addresses
typedef in_addr_t ip_type;
const ip_type none = INADDR_NONE;
const ip_type any = INADDR_ANY;
const ip_type loopback = INADDR_LOOPBACK;


// return local machine host name
const char *hostname (void);

// return IP address corresponding to interface name iface
in_addr_t lookup_iface (const char *iface);


inline const char *
ipstr (struct in_addr in)
{
  return inet_ntoa(in);
}


inline const char *
hoststr (struct in_addr in)
{
  hostent *hp = gethostbyaddr((char *)&in, sizeof(in), socket_base::inet);
  return hp && hp->h_name ? hp->h_name : ipstr(in);
}


}   // namespace net
}   // namespace bits
}   // namespace cool


#endif  /* cool_net_bits_inet_address_hpp_included */
