/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_net_bits_socket_base_hpp_included
#define cool_net_bits_socket_base_hpp_included


#ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#  include <sys/socket.h>
#endif


namespace cool {
namespace bits {
namespace net {


#if defined(COOL_POSIX)
typedef int socket_type;

#ifndef SS_SOCKLEN_T
// autoconf couldn't find correct type, let's hope the best...
#  warning No SS_SOCKLEN_T set, trying to use 'unsigned int'
#  define SS_SOCKLEN_T unsigned int
#endif
typedef SS_SOCKLEN_T socklen_type;

#else // COOL_POSIX
#  error Not implemented yet
#endif


enum family_type     { family_ = -1 };
enum style_type      { style_ = -1 };
enum shutmode_type   { shutmode_ = -1 };
enum io_flags_type   { io_flags_ = -1 };
enum option_type     { option_ = -1 };
enum level_type      { level_ = -1 };
enum event_type      { event_ = -1 };


struct linger_type
{
  int l_onoff;
  int l_linger;
  linger_type (int onoff_, int linger_): l_onoff(onoff_), l_linger(linger_) {}
};


inline io_flags_type
operator & (io_flags_type __a, io_flags_type __b)
{ return io_flags_type(static_cast<int>(__a) & static_cast<int>(__b)); }

inline io_flags_type
operator | (io_flags_type __a, io_flags_type __b)
{ return io_flags_type(static_cast<int>(__a) | static_cast<int>(__b)); }

inline io_flags_type
operator ^ (io_flags_type __a, io_flags_type __b)
{ return io_flags_type(static_cast<int>(__a) ^ static_cast<int>(__b)); }

inline io_flags_type
operator &= (io_flags_type &__a, io_flags_type __b)
{ return __a = __a & __b; }

inline io_flags_type
operator |= (io_flags_type &__a, io_flags_type __b)
{ return __a = __a | __b; }

inline io_flags_type
operator ^= (io_flags_type &__a, io_flags_type __b)
{ return __a = __a ^ __b; }

inline io_flags_type
operator ~ (io_flags_type __a)   
{ return io_flags_type(~static_cast<int>(__a)); }


inline event_type
operator & (event_type __a, event_type __b)
{ return event_type(static_cast<int>(__a) & static_cast<int>(__b)); }

inline event_type
operator | (event_type __a, event_type __b)
{ return event_type(static_cast<int>(__a) | static_cast<int>(__b)); }

inline event_type
operator ^ (event_type __a, event_type __b)
{ return event_type(static_cast<int>(__a) ^ static_cast<int>(__b)); }

inline event_type
operator &= (event_type &__a, event_type __b)
{ return __a = __a & __b; }

inline event_type
operator |= (event_type &__a, event_type __b)
{ return __a = __a | __b; }

inline event_type
operator ^= (event_type &__a, event_type __b)
{ return __a = __a ^ __b; }

inline event_type
operator ~ (event_type __a)
{ return event_type(~static_cast<int>(__a)); }


}   // namespace net
}   // namespace bits
}   // namespace cool


#endif  /* cool_net_bits_socket_base_hpp_included */
