/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_net_exception_hpp_included
#define cool_net_exception_hpp_included

/**
 * \file net_exception.hpp
 * \author Sven Suursoho
 *
 * \brief Class(es): cool::already_bound, cool::connection_error,
 * cool::connection_refused, cool::host_not_found, cool::iface_not_found,
 * cool::in_use_error, cool::network_error, cool::service_not_found
 *
 * In module \ref cool_net.
 */

#include <cool/cool.hpp>
#include <cool/exception/exception.hpp>


namespace cool {
/** \addtogroup cool_net */
/** \{ */


/**
 * \class already_bound
 * \brief Thrown whenever attempt bind already bounded socket.
 */
EXCEPTION_CLASS(already_bound, exception);

/**
 * \class connection_error
 * \brief Thrown whenever connection is reset or broken.
 */
EXCEPTION_CLASS(connection_error, exception);

/**
 * \class connection_refused
 * \brief Thrown whenever attempt to connect fails.
 */
EXCEPTION_CLASS(connection_refused, exception);

/**
 * \class host_not_found
 * \brief Thrown whenever resolving hostname to IP-address fails.
 */
EXCEPTION_CLASS(host_not_found, exception);

/**
 * \class iface_not_found
 * \brief Thrown during cool::inet_address::iface() if interface name can't be
 * resolved.
 */
EXCEPTION_CLASS(iface_not_found, exception);

/**
 * \class in_use_error
 * \brief Thrown whenever socket attempts to listen port that is already used.
 */
EXCEPTION_CLASS(in_use_error, exception);

/**
 * \class network_error
 * \brief Thrown whenever network or host is unreachable.
 */
EXCEPTION_CLASS(network_error, exception);

/**
 * \class service_not_found
 * \brief Thrown whenever resolving service name to port number fails.
 */
EXCEPTION_CLASS(service_not_found, exception);


/** \} */
}   // namespace cool


#endif  /* cool_net_exception_hpp_included */
