/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_net_socket_pair_hpp_included
#define cool_net_socket_pair_hpp_included

/**
 * \file socket_pair.hpp
 * \author Sven Suursoho
 *
 * \brief Class(es): cool::socket_pair
 *
 * In module \ref cool_net.
 */

#include <cool/cool.hpp>
#include <cool/util/noncopyable.hpp>
#include <cool/net/socket.hpp>
#include "bits/socket_pair.hpp"
#include utility_header


namespace cool {
/** \addtogroup cool_net */
/** \{ */


/**
 * \brief Pair of connected sockets
 *
 * This class creates and holds pair of an unnamed connected sockets in
 * specified family and style type. The two sockets are indistinguishable.
 *
 * \note On Linux, the only supported family is socket::local.
 */
template <typename family>
class COOL_API socket_pair
    : private noncopyable
    , public STD_ pair<typename family::socket, typename family::socket>
{
  public:

      /** \brief Instantiated socket type */
      typedef typename family::socket socket_type;


      /**
       * \brief Create new pair of connected sockets.
       *
       * If non-supported family is specified as template parameter,
       * cool::not_supported_error exception is thrown.
       */
      socket_pair (cool::socket::style_type style = cool::socket::def::style) {
          bits::net::socket_pair(this->first, this->second,
                  family::family, style);
      }


      /** \brief Close both sockets */
      ~socket_pair () {
          this->first.close();
          this->second.close();
      }
};


/** \} */
}   // namespace cool


#endif  /* cool_net_socket_pair_hpp_included */
