/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_thread_hpp_included
#define cool_thread_hpp_included

/**
 * \defgroup cool_thread Threading
 * \ingroup cool
 * \{
 * Threading allows programs to execute as multiple, asynchronous independent
 * threads-of-execution. Each thread has it's own machine state including
 * program instruction counter and registers. Multithreading providers several
 * advantages:
 *  - application that could block waiting for some external event can
 *    continue to respond if the possibly blocking operation is ran in
 *    different thread
 *  - well-designed multithreaded applications may execute faster than
 *    single-threaded applications
 *  - etc
 *
 * Header(s):
 *  - \ref thread/exception.hpp
 *  - \ref thread/mutex.hpp
 *  - \ref thread/recursive_mutex.hpp
 *  - \ref thread/condition.hpp
 *  - \ref thread/thread.hpp
 *  - \ref thread/thread_list.hpp
 *  - \ref thread/tss.hpp
 *  - \ref thread/barrier.hpp
 *  - \ref thread/once.hpp
 *  - \ref thread/queue.hpp
 *  - \ref thread/blocking_queue.hpp
 *  - \ref thread/bounded_queue.hpp
 *  - \ref thread/locked_queue.hpp
 *
 * \note Design and lots of code for cool::mutex and cool::condition is taken
 * from excellent boost::thread library by William E. Kempf. See
 * http://www.boost.org/doc/html/threads.html<br>
 * Don't even know, why to bother reinventing wheel again and again. Oh, now I
 * remember -- I like name COOL better :)
 *
 * \}
 */

#include <cool/thread/exception.hpp>
#include <cool/thread/mutex.hpp>
#include <cool/thread/recursive_mutex.hpp>
#include <cool/thread/condition.hpp>
#include <cool/thread/thread.hpp>
#include <cool/thread/thread_list.hpp>
#include <cool/thread/tss.hpp>
#include <cool/thread/barrier.hpp>
#include <cool/thread/once.hpp>
#include <cool/thread/queue.hpp>
#include <cool/thread/blocking_queue.hpp>
#include <cool/thread/bounded_queue.hpp>
#include <cool/thread/locked_queue.hpp>


#endif  /* cool_thread_hpp_included */
