/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_thread_barrier_hpp_included
#define cool_thread_barrier_hpp_included

/**
 * \file barrier.hpp
 * \author Sven Suursoho
 *
 * \brief Class(es): cool::barrier
 *
 * In module \ref cool_thread.
 */

#include <cool/cool.hpp>
#include <cool/thread/mutex.hpp>
#include <cool/thread/condition.hpp>

#if defined(barrier)
#  undef barrier
#endif


namespace cool {
/** \addtogroup cool_thread */
/** \{ */


/**
 * \brief Barrier synchronization primitive.
 *
 * Barrier is used to force specified set of threads to be suspended until all
 * of them have called wait() i.e. restart all threads in set at certain
 * point.
 */
class COOL_API barrier
{
  public:

      /** \brief Create barrier with thread \a count. */
      barrier (unsigned int count)
          : threshold_(count), count_(count), generation_(0)
      {
          assert(count > 0);
      }


      /** \brief Destroy barrier */
      ~barrier () { }


      /**
       * \brief Wait until \a count threads have called wait().
       *
       * All threads except one receive \c false as return value and one
       * receives \c true. Which thread receives \c true is not defined.
       *
       * After all threads are restarted, this barrier is again in initial
       * state.
       */
      bool wait ();


  private:

      mutex mutex_;
      condition cond_;
      unsigned int threshold_;
      unsigned int count_;
      unsigned int generation_;
};


/** \} */
}   // namespace cool


#endif  /* cool_thread_barrier_hpp_included */
