/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_thread_bits_condition_hpp_included
#define cool_thread_bits_condition_hpp_included

#if defined(HAVE_PTHREAD_H)
#  include <pthread.h>
#endif
#include <cool/thread/exception.hpp>


namespace cool {
namespace bits {
namespace thread {


class condition
{
  public:

      condition () {
          if (::pthread_cond_init(&cond_, 0) != 0)
              raise<resource_error>(COOL_LOCUS);
      }


      ~condition () {
          int r = 0;
          r = ::pthread_cond_destroy(&cond_);
          assert(r == 0);
      }


      void wait (::pthread_mutex_t *pmutex) {
          int r = 0;
          r = ::pthread_cond_wait(&cond_, pmutex);
          assert(r == 0);
      }


      bool wait (::pthread_mutex_t *pmutex, const cool::time &t) {
          struct timespec timeout = { t.sec, t.nsec };
          int r = ::pthread_cond_timedwait(&cond_, pmutex, &timeout);
          assert(r == 0 || r == ETIMEDOUT);
          return r != ETIMEDOUT;
      }


      void notify_one () {
          int r = 0;
          r = ::pthread_cond_signal(&cond_);
          assert(r == 0);
      }


      void notify_all () {
          int r = 0;
          r = ::pthread_cond_broadcast(&cond_);
          assert(r == 0);
      }


  private:

      ::pthread_cond_t cond_;
};


}   // namespace thread
}   // namespace bits
}   // namespace cool


#endif  /* cool_thread_bits_condition_hpp_included */
