/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_thread_bits_thread_hpp_included
#define cool_thread_bits_thread_hpp_included

#if defined(HAVE_PTHREAD_H)
#  include <pthread.h>
#endif


namespace cool {
namespace bits {
namespace thread {


struct thread
{
  pthread_t thread_;


  thread (): thread_(pthread_self()) { }


  static void create (thread &t, void *(*fn)(void *), void *arg) {
      int r = pthread_create(&t.thread_, 0, fn, arg);
      switch (r) {
          case EAGAIN:
              raise<resource_error>(COOL_LOCUS);

          case EPERM:
              raise<permission_error>(COOL_LOCUS);

          default:
              assert(r == 0);
      }
  }


  void detach () {
      int r = 0;
      r = pthread_detach(thread_);
      assert(r == 0);
  }


  void join () {
      int r = 0;
      r = pthread_join(thread_, 0);
      assert(r == 0);
  }


  bool equal (const thread &o) const {
      return pthread_equal(thread_, o.thread_) != 0;
  }


  static void sleep (const cool::time &t) {
#if defined(HAVE_PTHREAD_DELAY_NP)

      // XXX not tested
      struct timespec ts = t;
      int r = pthread_delay_np(&ts);
      assert(r == 0);

#elif defined(HAVE_NANOSLEEP)

      struct timespec ts = t, remain;
      while (nanosleep(&ts, &remain) == -1 && errno == EINTR) {
          ts.tv_sec = remain.tv_sec;
          ts.tv_nsec = remain.tv_nsec;
      }

#else
#  error No function to sleep (use select() or smth to implement)
#endif
  }


  static void yield () {
#if defined(SS_THREAD_YIELD)
      int r = 0;
      r = SS_THREAD_YIELD();
      assert(r == 0);
#else
      sleep(cool::time(0));
#endif
  }
};


}   // namespace thread
}   // namespace bits
}   // namespace cool


#endif  /* cool_thread_bits_thread_hpp_included */
