/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_thread_bits_tss_hpp_included
#define cool_thread_bits_tss_hpp_included


#if defined(HAVE_PTHREAD_H)
#  include <pthread.h>
#endif
#include <cool/thread/mutex.hpp>
#include vector_header


namespace cool {
namespace bits {
namespace thread {


typedef STD_ vector<void *> tss_slots;


struct tss_data_t
{
  mutex mutex_;
  pthread_key_t key_;
  STD_ vector<void (*)(void*)> cleanup_handlers_;
};


extern tss_data_t *tss_data;
tss_slots *get_slots (bool alloc);


class tss
{
  public:

      tss (void (*clean)(void *)) {
          assert(clean != 0);
          init(clean);
      }


      void *get () const {
          tss_slots *slots = get_slots(false);
          if (!slots)
              return 0;

          if (slot_ >= slots->size())
              return 0;

          return (*slots)[slot_];
      }


      void set (void *value) {
          tss_slots *slots = get_slots(true);
          if (!slots)
              raise<resource_error>(COOL_LOCUS);

          if (slot_ >= slots->size()) {
              try {
                  slots->resize(slot_ + 1);
              }
              catch (...) {
                  raise<resource_error>(COOL_LOCUS);
              }
          }

          (*slots)[slot_] = value;

      }


      void cleanup (void *value) {
          mutex::scoped_lock guard(tss_data->mutex_);
          (*tss_data->cleanup_handlers_[slot_])(value);
      }


  private:

      void init (void (*clean)(void *));

      unsigned int slot_;
};


}   // namespace thread
}   // namespace bits
}   // namespace cool


#endif  /* cool_thread_bits_tss_hpp_included */
