/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_thread_once_hpp_included
#define cool_thread_once_hpp_included

/**
 * \file once.hpp
 * \author Sven Suursoho
 *
 * \brief Function(s): cool::call_once
 *
 * In module \ref cool_thread.
 */

#include <cool/cool.hpp>
#include "bits/once.hpp"


namespace cool {
/** \addtogroup cool_thread */
/** \{ */


/**
 * \brief Flag, storing information whether associated function is already
 * called.
 */
typedef bits::thread::once_flag once_flag;


/** \brief Static initializer for \a once_flag. */
#define COOL_ONCE_INIT COOL_BITS_ONCE_INIT


/**
 * \brief Once-only initialization.
 *
 * Purpose of this function is to ensure that a piece of initialization code
 * is executed at most once. The \a flag argument must be a static or extern
 * variable statically initialized to COOL_ONCE_INIT.
 *
 * The first time call_once is called with a given \a flag, it calls \a func
 * and changes \a flag, so that next time call_once knows not to call \a func.
 */
void COOL_API call_once (void (*func)(), once_flag &flag);


/** \} */
}   // namespace cool


#endif  /* cool_thread_once_hpp_included */
