/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_thread_queue_hpp_included
#define cool_thread_queue_hpp_included

/**
 * \file queue.hpp
 * \author Sven Suursoho
 *
 * \brief Class(es): cool::queue
 *
 * In module \ref cool_thread.
 */

#include <cool/cool.hpp>
#include <cool/util/noncopyable.hpp>


namespace cool {
/** \addtogroup cool_thread */
/** \{ */


/**
 * \brief Interface for collection of object (similar to STL collections).
 *
 * Threads can use classes inherited from queue to pass messages to each other
 * in synchronized manner.
 */
template <typename T>
class COOL_API queue: private noncopyable
{
  public:

      /** \brief Type of items that can be inserted into queue */
      typedef T value_type;


      /** \brief Create new empty queue */
      queue () {}


      /** \brief Destroy queue */
      virtual ~queue () {}


      /** \brief Add an item to queue */
      virtual void push (const value_type &item) = 0;


      /** \brief Retrieve and remove item from queue */
      virtual value_type pop () = 0;


      /** \brief Return number of items in queue */
      virtual size_t size () = 0;


      /** \brief Return true if collection has no items */
      virtual bool empty () {
          return size() == 0;
      }
};


/** \} */
}   // namespace cool


#endif  /* cool_thread_queue_hpp_included */
