/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_time_bits_time_hpp_included
#define cool_time_bits_time_hpp_included


#if defined(HAVE_STRING_H)
#  include <string.h>
#endif 
#if defined(HAVE_SYS_TIME_H)
#  include <sys/time.h>
#endif 
#if defined(HAVE_TIME_H)
#  include <time.h>
#endif


namespace cool {
namespace bits {
namespace time {


static const int nsec_per_sec = 1000000000;
static const int nsec_per_msec = 1000000;
static const int usec_per_sec = 1000000;
static const int nsec_per_usec = 1000;
static const int usec_per_msec = 1000;
static const int msec_per_sec = 1000;
  
  
typedef unsigned long sec_type;
typedef unsigned long nsec_type;


char *as_iso8601 (sec_type sec, char *buf, size_t bufsz);
sec_type parse_iso8601 (const char *buf, char **eptr);


inline sec_type
get (nsec_type *nsec)
{
#if defined(HAVE_GETTIMEOFDAY)
  struct timeval t;
  gettimeofday(&t, 0);
  if (nsec)
      *nsec = t.tv_usec*nsec_per_usec;
  return t.tv_sec;
#elif defined(HAVE_CLOCK_GETTIME)
  struct timespec t;
  clock_gettime(CLOCK_REALTIME, &t);
  if (nsec)
      *nsec = t.tv_nsec;
  return t.tv_sec;
#elif defined(HAVE_TIME)
  if (nsec)
      *nsec = 0;
  return time(0);
#else
#  error No function to get time
  return 0;
#endif
}


inline struct timespec
as_timespec (sec_type sec, nsec_type nsec)
{
  nsec_type nsec_;
  sec_type sec_ = get(&nsec_);

  struct timespec ts = { 0, 0 };
  if (sec_ >= sec && nsec_ >= nsec)
      return ts;

  ts.tv_sec = sec - sec_;
  if ((ts.tv_nsec = nsec - nsec_) < 0) {
      --ts.tv_sec;
      ts.tv_nsec += nsec_per_sec;
  }

  if (ts.tv_nsec > nsec_per_sec) {
      // nanosleep requires nanosec to be in range 0..999999999
      ts.tv_sec += ts.tv_nsec/nsec_per_sec;
      ts.tv_nsec %= nsec_per_sec;
  }

  return ts;
}


inline struct timeval
as_timeval (sec_type sec, nsec_type nsec)
{
  nsec_type nsec_;
  sec_type sec_ = get(&nsec_);

  struct timeval tv = { 0, 0 };
  if (sec_ >= sec && nsec_ >= nsec)
      return tv;

  tv.tv_sec = sec - sec_;
  tv.tv_usec = (int)(nsec - nsec_)/nsec_per_usec;
  if (tv.tv_usec < 0) {
      --tv.tv_sec;
      tv.tv_usec += usec_per_sec;
  }

  return tv;
}


}   // namespace time
}   // namespace bits
}   // namespace cool


#endif  /* cool_time_bits_time_hpp_included */
