/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_util_bits_convert_hpp_included
#define cool_util_bits_convert_hpp_included


#if defined(HAVE_LIMITS_H)
#  include <limits.h>
#endif
#include <cool/exception/os_error.hpp>


namespace cool {
namespace bits {
namespace convert {


// char
const signed long char_min = SCHAR_MIN;
const signed long char_max = SCHAR_MAX;
const unsigned long uchar_max = UCHAR_MAX;

// short
const signed long short_min = SHRT_MIN;
const signed long short_max = SHRT_MAX;
const unsigned long ushort_max = USHRT_MAX;

// int
const signed long int_min = INT_MIN;
const signed long int_max = INT_MAX;
const unsigned long uint_max = UINT_MAX;


inline bool
bool_ (const char *string, char ** = 0)
{
  return
      string[0] == 'T' || string[0] == 't' ||
      string[0] == 'Y' || string[0] == 'y' ||
      (string[0] > '0' && string[0] <= '9');
}


inline unsigned long
unsigned_long_ (const char *string, char **eptr = 0)
{
  cool::os_error::last(0);
  char *eptr_;
  unsigned long ul = strtoul(string, &eptr_, 10);

  if (cool::os_error::last() == ERANGE)
      // overflow
      raise<limit_error>(COOL_LOCUS);

  if (eptr_ == string)
      // no characters were converted
      raise<format_error>(COOL_LOCUS);

  if (eptr)
      *eptr = eptr_;

  return ul;
}

  
inline signed long
long_ (const char *string, char **eptr = 0)
{
  cool::os_error::last(0);
  char *eptr_; 
  signed long l = strtol(string, &eptr_, 10);
  
  if (cool::os_error::last() == ERANGE)
      // overflow
      raise<limit_error>(COOL_LOCUS);

  if (eptr_ == string)
      // no characters were converted
      raise<format_error>(COOL_LOCUS);

  if (eptr)
      *eptr = eptr_;

  return l;
}


}   // namespace convert
}   // namespace bits
}   // namespace cool


#endif  /* cool_util_bits_convert_hpp_included */
