/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#ifndef cool_util_less_str_hpp_included
#define cool_util_less_str_hpp_included

/**
 * \file less_str.hpp
 * \author Sven Suursoho
 *
 * \brief Class(es): cool::less_strcmp, cool::less_strcasecmp
 *
 * In module \ref cool_util.
 */

#include <cool/cool.hpp>

#if defined(HAVE_STRING_H)
#  include <string.h>
#else
#  if defined(HAVE_MEMORY_H)
#    include <memory.h>
#  endif
#endif


namespace cool {
/** \addtogroup cool_util */
/** \{ */


/**
 * \brief Case sensitive C-style string comparator
 * \see cool::less_strcasecmp
 */
struct COOL_API less_strcmp
{
  /** \brief Return \c true if \a s1 < \a s2 (alphabetically) */
  bool operator() (const char *s1, const char *s2) const
  { return ::strcmp(s1, s2) < 0; }
};


/**
 * \brief Case in-sensitive C-style string comparator
 * \see cool::less_strcmp
 */
struct COOL_API less_strcasecmp
{
  /** \brief Return \c true if \a s1 < \a s2 (alphabetically) */
  bool operator() (const char *s1, const char *s2) const
  { return ::strcasecmp(s1, s2) < 0; }
};


/** \} */
}   // namespace cool


#endif  /* cool_util_less_str_hpp_included */
