dnl Copyright (c) 2005, Sven Suursoho
dnl See LICENSE for details


dnl Enable C++ exception handling whenever possible
AC_DEFUN([SS_CXX_EXCEPTIONS],
[
    AC_REQUIRE([SS_CXX_PROGRAMMING])
    AC_LANG_SAVE
    AC_LANG(C++)

    dnl Strip -fno-exceptions flag if used
    optflags=$CXXFLAGS
    if test ! -z "$optflags"; then
        CXXFLAGS=""
        for opt in $optflags; do
            case $opt in
                *no-rtti*)
                    ;;
                *omit-frame-pointer*)
                    ;;
                *no-exceptions*)
                    ;;
                *)
                    CXXFLAGS="$CXXFLAGS $opt"
            esac
        done
    fi

    dnl Check for exception handling
    AC_CACHE_CHECK([whether ${CXX} supports -fhandle-exceptions],
        ss_cv_cxx_exception_flag, [
            echo 'void f(){}' > conftest.c
            if test -z "`${CXX} -fhandle-exceptions -c conftest.c 2>&1`"; then
                ss_cv_cxx_exception_flag=yes
                CXXFLAGS="$CXXFLAGS -fhandle-exceptions"
            else
                ss_cv_cxx_exception_flag=no
            fi
            rm -f conftest*
        ]
    )

    if test "$ss_cv_cxx_exception_flag" = "yes"; then
        ss_cv_cxx_exception_handling=yes
    else
        AC_CACHE_CHECK([whether ${CXX} supports exception handling],
            ss_cv_cxx_exception_handling,
            AC_TRY_COMPILE(
                [void f() { throw "abc"; }],
                [try { f(); } catch (char *) {}],
                ss_cv_cxx_exception_handling=yes,
                ss_cv_cxx_exception_handling=no
            )
        )
    fi

    if test "$ss_cv_cxx_exception_handling" = "yes"; then
        AC_DEFINE(HAVE_EXCEPTION_HANDLING, 1, [Support exception handling])
    fi

    AC_LANG_RESTORE
])
