dnl Copyright (c) 2005, Sven Suursoho
dnl See LICENSE for details


dnl Check for time functions
AC_DEFUN([SS_FUNC_GETTIME],
[
    AC_LANG_SAVE
    AC_LANG(C)


    dnl Check for time header files
    AC_CHECK_HEADERS([time.h sys/time.h])


    dnl Check for function to query system time
    AC_CACHE_CHECK([whether to use gettimeofday, clock_gettime or time],
        [ss_cv_func_gettime],
        [
            AC_TRY_COMPILE([
#include <stdlib.h>
#if defined(HAVE_SYS_TIME_H)
#  include <sys/time.h>
#endif
                ],
                [struct timeval t; gettimeofday(&t, NULL);],
                [ss_cv_func_gettime=gettimeofday],
                [
                    AC_TRY_COMPILE([
#if defined(HAVE_TIME_H)
#  include <time.h>
#endif
                        ],
                        [struct timespec t; clock_gettime(CLOCK_REALTIME, &t);],
                        [ss_cv_func_gettime=clock_gettime],
                        [
                            AC_TRY_COMPILE([
#if defined(HAVE_TIME_H)
#  include <time.h>
#endif
                                ],
                                [time_t t; time(&t);],
                                [ss_cv_func_gettime=time],
                                [ss_cv_func_gettime=none]
                            )
                        ]
                    )
                ]
            )
        ]
    )

    if test $ss_cv_func_gettime = gettimeofday; then
        AC_DEFINE(HAVE_GETTIMEOFDAY, 1,
            [gettimeofday() for querying system time])
    elif test $ss_cv_func_gettime = clock_gettime; then
        AC_DEFINE(HAVE_CLOCK_GETTIME, 1,
            [clock_gettime() for querying system time])
        LIBS="$LIBS -lrt"
    elif test $ss_cv_func_gettime = time; then
        AC_DEFINE(HAVE_TIME, 1, [time() for querying system time])
    fi


    dnl Check whether we have localtime_r
    AC_CACHE_CHECK([whether we have localtime_r],
        [ss_cv_func_localtime_r],
        [
            AC_TRY_COMPILE([
#if defined(HAVE_TIME_H)
#  include <time.h>
#elif defined(HAVE_SYS_TIME_H)
#  include <sys/time.h>
#else
/* Hope for best */
#  include <stdlib.h>
#endif
                ],
                [localtime_r(0, 0);],
                [ss_cv_func_localtime_r=yes],
                [ss_cv_func_localtime_r=no]
            )
        ]
    )

    if test $ss_cv_func_localtime_r = yes; then
        AC_DEFINE(HAVE_LOCALTIME_R, 1, [thread-safe version of localtime])
    fi

    AC_LANG_RESTORE
])
