dnl Copyright (c) 2005, Sven Suursoho
dnl See LICENSE for details


dnl Check for socket multiplexor function
AC_DEFUN([SS_SOCKET_SELECT_FUNC],
[
    AC_LANG_SAVE
    AC_LANG(C)

    dnl check for headers
    AC_CHECK_HEADERS([ \
        stdlib.h \
        sys/epoll.h \
        sys/poll.h \
        sys/select.h \
        sys/time.h \
        sys/types.h \
        unistd.h \
    ])

    dnl check for epoll_*()
    dnl Checking existence of epoll_create is not enough as function might
    dnl exist but it always returns "not supported" error
    AC_MSG_CHECKING([check for epoll family functions])
    AC_TRY_RUN([
#include <stdlib.h>
#include <sys/epoll.h>
int main (void) { exit(epoll_create(1)>-1 ? 0 : 1); }
        ],
        [
            ss_socket_epoll_func=yes
            AC_DEFINE(SS_HAVE_EPOLL, 1, [multiplexor function])
        ],
        [ss_socket_epoll_func=no]
    )
    AC_MSG_RESULT([$ss_socket_epoll_func])


    dnl check for poll()
    AC_MSG_CHECKING([check for poll()])
    AC_TRY_COMPILE([
#include <stdlib.h>
#include <sys/poll.h>
        ],
        [return poll(NULL, 0, 0);],
        [
            ss_socket_poll_func=yes
            AC_DEFINE(SS_HAVE_POLL, 1, [multiplexor function])
        ],
        [ss_socket_poll_func=no]
    )
    AC_MSG_RESULT([$ss_socket_poll_func])


    dnl Check for select
    AC_MSG_CHECKING([check for select()])
    AC_TRY_COMPILE([
#include <stdlib.h>
#include <sys/poll.h>
        ],
        [return select(0, NULL, NULL, NULL, NULL)],
        [
            ss_socket_select_func=yes
            AC_DEFINE(SS_HAVE_SELECT, 1, [multiplexor function])
        ],
        [ss_socket_select_func=no]
    )
    AC_MSG_RESULT([$ss_socket_select_func])

    AC_LANG_RESTORE
])

