/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#include <cool/config/ini_parser.hpp>

#if defined(HAVE_CTYPE_H)
#  include <ctype.h>
#endif


namespace cool {


bool
ini_parser::next (STD_ istream &is)
{
  reset();
  skip(is);

  int got;
  while ((got = is.getline(line + key_size, size - key_size).gcount()) != 0) {
      key_size += got;

      if (!is.fail() && got > 1) {
          --key_size;

          // skip tail whitespaces
          while (isspace(line[key_size - 1]))
              --key_size;

          if (line[key_size - 1] == escape) {
              // value continues on next line
              is >> STD_ ws;
              --key_size;
              continue;
          }

          line[key_size] = '\0';
          if (!good(line))
              // indicates that we shouldn't read more from config stream
              break;

          parse_line();
          return true;
      }

      do_alloc(size << 1);
      is.clear();
  }

  return false;
}


void
ini_parser::do_alloc (int new_size)
{
  char *p = (char *)(line ? realloc(line, new_size) : malloc(new_size));
  if (!p)
      raise<resource_error>(COOL_LOCUS);

  line = p;
  size = new_size;
}


void
ini_parser::skip (STD_ istream &is)
{
  while (true) {
      is >> STD_ ws;
      if (is.peek() != comment)
          return;
      is.ignore(INT_MAX, '\n');
  }
}


void
ini_parser::parse_line ()
{
  char *end = (value_ptr = line) + key_size;
  while (value_ptr < end && *value_ptr != assign)
      ++value_ptr;
  if (*value_ptr != assign)
      raise<format_error>(COOL_LOCUS);

  // extract key
  end = value_ptr++;
  value_size = key_size - (end - line) - 1;
  while (end > line && isspace(*(end - 1)))
      --end;
  if ((key_size = end - line) == 0)
      raise<format_error>(COOL_LOCUS);
  *end = '\0';

  // extract value
  end = value_ptr + value_size;
  while (value_ptr < end && isspace(*value_ptr))
      ++value_ptr;
  value_size = end - value_ptr;

  // unescape value
  char *ptr = end = value_ptr;
  while (*end) {
      if (*end != escape) {
          *ptr++ = *end++;
          continue;
      }

      switch (*++end) {
          case 'a': *ptr++ = '\a'; break;
          case 'b': *ptr++ = '\b'; break;
          case 'f': *ptr++ = '\f'; break;
          case 'n': *ptr++ = '\n'; break;
          case 'r': *ptr++ = '\r'; break;
          case 't': *ptr++ = '\t'; break;
          case 'v': *ptr++ = '\v'; break;
          default: *ptr++ = *end;
      }
      ++end;
  }

  value_size = ptr - value_ptr;
  *ptr = '\0';
}


}  // namespace cool
