/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#include <cool/net/inet_address.hpp>

#ifdef HAVE_SYS_IOCTL_H
#  include <sys/ioctl.h>
#endif
#ifdef HAVE_NET_IF_H
#  include <net/if.h>
#endif
#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#endif


namespace cool {


inet_address & 
inet_address::host (const char *host_)
{
  if (!host_ || !*host_ || *host_ == '*') {
      // Use local with interface any
      s_addr = htonl(any);
      return *this;
  }

  // try textual IP address
  if ((s_addr = inet_addr(host_)) != INADDR_NONE)
      // textual IP address
      return *this;

  // try resolve host name
  hostent *hp = gethostbyname(host_);
  if (hp) {
      memcpy(&s_addr, hp->h_addr, hp->h_length);
      return *this;
  }

  // couldn't convert name -> IP
  raise<host_not_found>(COOL_LOCUS);
  return *this;
}


namespace bits {
namespace net {
  
const char *
hostname ()
{
  static char host[inet_address::host_length] = { '\0' };

  if (!*host) {
      // get host name first time
      gethostname(host, sizeof(host));
      host[sizeof(host) - 1] = '\0';
  }

  return host;
}


in_addr_t
lookup_iface (const char *iface)
{
#ifdef HAVE_NET_IF_H
  struct {
      char ifrn_name[IFNAMSIZ];
      struct sockaddr_in ifru_addr;
  } ifreq;

  strncpy(ifreq.ifrn_name, iface, sizeof(ifreq.ifrn_name));
  int s = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
  int r = ::ioctl(s, SIOCGIFADDR, &ifreq);
  ::close(s);

  if (r != -1)
      return ifreq.ifru_addr.sin_addr.s_addr;
#endif

  return INADDR_NONE;
}


}   // namespace net
}   // namespace bits
}   // namespace cool
