/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#include <cool/net/inet_socket.hpp>
#include <cool/net/exception.hpp>

#if defined(HAVE_NETINET_TCP_H)
#  include <netinet/tcp.h>
#endif


namespace cool {


const socket::family_type inet::family = socket::inet;


inet::address::port_type
inet::address::service (const char *servname, const char *proto)
{
  servent *sp = getservbyname(servname, proto);
  if (!sp)
      raise<service_not_found>(COOL_LOCUS);
  return ntohs(sp->s_port);
}


inet::address &
inet::address::operator = (const char *as_string)
{
  if (!as_string || !*as_string) {
      host(inet_address::any);
      port(0);
      return *this;
  }

  // extract port
  char *p = strchr(as_string, ':');
  if (p) {
      if (p[1]) {
          try {
              // specified as integer
              port(as<inet::address::port_type>(p + 1, 0));
          }
          catch (format_error &e) {
              // specified as service name
              port(inet::address::service(p + 1));
          }
      }

      // hostname without port
      char h[inet_address::host_length];
      strncpy(h, as_string, p - as_string);
      h[p - as_string] = '\0';
      host(inet_address(h));
  }
  else
      // whole string as only hostname
      host(inet_address(as_string));

  return *this;
}


inet::address
inet::socket::local () const
{
  inet::address a;
  socklen_type len = a.size();

  if (::getsockname(s_, a.addr(), &len) == -1)
      bits::net::throw_last("getsockname", COOL_LOCUS);

  return a;
}


inet::address
inet::socket::peer () const
{
  inet::address a;
  socklen_type len = a.size();

  if (::getpeername(s_, a.addr(), &len) == -1)
      bits::net::throw_last("getpeername", COOL_LOCUS);

  return a;
}


bool
inet::socket::tcpnodelay ()
{
  struct protoent *proto = ::getprotobyname("tcp");
  if (!proto)
      return false;

  int o;
  getopt(option_type(TCP_NODELAY), &o, sizeof(o), level_type(proto->p_proto));
  return o;
}


inet::socket &
inet::socket::tcpnodelay (bool on)
{
  struct protoent *proto = ::getprotobyname("tcp");
  if (!proto)
      return *this;

  int o = on;
  setopt(option_type(TCP_NODELAY), &o, sizeof(o), level_type(proto->p_proto));
  return *this;
}


}   // namespace cool
