/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#include <cool/process/process.hpp>


namespace cool {
namespace bits {
namespace process {

      
int
spawn (const char *exe, const char **argv)
{
  assert(exe != 0);
      
  // if arguments not specified, create our own list with exe as first item
  const char *default_argv[] = { exe, 0 };
  if (!argv || !argv[0])
      argv = default_argv;
  
  // exe name and first parameter must be same
  assert(::strcmp(argv[0], exe) == 0);

  pid_type pid = ::vfork();
  if (pid == -1)
      raise<resource_error>(COOL_LOCUS);
  else if (!pid) {
      ::execvp(exe, (char **)argv);
      _exit(-1);
  }

  int status;
  wait_pid(pid, &status);
  return status;
}
      
      
void
wait_error (const char *func)
{
  cool::os_error::error_type e = cool::os_error::last();
  switch (e) {
      case ECHILD:
          raise<not_found_error>(COOL_LOCUS);

      case EFAULT: 
      case EINVAL:
          raise<invalid_value_error>(COOL_LOCUS);
  
      case EINTR:
          raise<interrupted>(COOL_LOCUS);
  
      default:
          raise(cool::os_error(func), COOL_LOCUS);
  }
}


}   // namespace process
}   // namespace bits
}   // namespace cool
