/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#include <cool/thread/thread.hpp>
#include <cool/thread/mutex.hpp>
#include <cool/thread/condition.hpp>


namespace cool {


struct thread_param
{
  enum { struct_func_begin, ptr_func_begin, running };

  thread_param (thread::handler *f)
      : state_(struct_func_begin), sf_(f) {}

  thread_param (void (*f)(void *), void *a)
      : state_(ptr_func_begin), pf_(f), arg_(a) {}

  void wait () {
      mutex::scoped_lock guard(mutex_);
      while (state_ != running)
          cond_.wait(guard);
  }

  void started () {
      mutex::scoped_lock guard(mutex_);
      state_ = running;
      cond_.notify_one();
  }

  int state_;
  thread::handler *sf_;
  void (*pf_)(void *);
  void *arg_;
  mutex mutex_;
  condition cond_;
};


extern "C"
{
  static void *
  thread_func (void *param)
  {
    try {
        thread_param *p = static_cast<thread_param *>(param);
        switch (p->state_) {
            case thread_param::struct_func_begin: {
                thread::handler *sf = p->sf_;
                p->started();
                (*sf)();
                break;
            }

            case thread_param::ptr_func_begin: {
                void *arg = p->arg_;
                void (*pf)(void *) = p->pf_;
                p->started();
                pf(arg);
                break;
            }
        }
    }
    catch (...) {
        assert(!"Exception escaped from thread!");
    }

    return 0;
  }
}


thread::thread (thread::handler *h)
    : joinable_(true)
{
  thread_param p(h);
  bits::thread::thread::create(thread_, thread_func, &p);
  p.wait();
}


thread::thread (void (*h)(void *), void *arg)
    : joinable_(true)
{
  thread_param p(h, arg);
  bits::thread::thread::create(thread_, thread_func, &p);
  p.wait();
}


}   // namespace cool
