/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#include <cool/thread/tss.hpp>
#include <cool/thread/once.hpp>
#include memory_header


namespace cool {
namespace bits {
namespace thread {

tss_data_t *tss_data = 0;
once_flag tss_data_once = COOL_ONCE_INIT;


extern "C"
void
cleanup_slots (void *p)
{
  tss_slots *slots = static_cast<tss_slots *>(p);
  for (tss_slots::size_type i = 0;  i < slots->size();  ++i) {
      mutex::scoped_lock guard(tss_data->mutex_);
      (*tss_data->cleanup_handlers_[i])((*slots)[i]);
      (*slots)[i] = 0;
  }
}


void
init_tss_data ()
{
  STD_ auto_ptr<tss_data_t> t(new tss_data_t);
  if (pthread_key_create(&t->key_, &cleanup_slots) != 0)
      return;

  // intentional leak: to ensure that mtuex in global data is available when
  // cleanup handlers are run
  tss_data = t.release();
}


tss_slots *
get_slots (bool alloc)
{
  tss_slots *slots =
      static_cast<tss_slots *>(pthread_getspecific(tss_data->key_));
  if (!slots && alloc) {
      STD_ auto_ptr<tss_slots> t(new tss_slots);

      if (pthread_setspecific(tss_data->key_, t.get()) != 0)
          return 0;

      slots = t.release();
  }

  return slots;
}


void
tss::init (void (*clean)(void *))
{
  call_once(&init_tss_data, tss_data_once);
  if (!tss_data)
      raise<resource_error>(COOL_LOCUS);

  mutex::scoped_lock guard(tss_data->mutex_);
  try {
      tss_data->cleanup_handlers_.push_back(clean);
      slot_ = tss_data->cleanup_handlers_.size() - 1;
  }
  catch (...) {
      raise<resource_error>(COOL_LOCUS);
  }
}


}   // namespace thread
}   // namespace bits
}   // namespace cool
