/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>

#if defined(INCLUDE_MODULE)
#  include <cool/config.hpp>
#else
#  include <cool/config/config.hpp>
#  include <cool/config/ini_parser.hpp>
#endif

#include iostream_header

#if defined(sstream_header)
#  include sstream_header
#elif defined(strstream_header)
#  include strstream_header
#endif


const char conf_content[] =
    "#\n"
    "# comment\n"
    "#\n"
    "\n"
    "\tk1 = v1\n"
    "k2\t=v2\n"
    "k3=\tv3\n"
    "k4 = \\\n"
    "    v4\n"
    "k5\t=\t\\\n"
    "    v\\\n"
    "    5\n"
    "#\n"
    "# end\n"
    "#";

int
main (int argc, char *argv[])
{
  try {
#if defined(sstream_header)
      STD_ stringstream ins(conf_content);
#elif defined(strstream_header)
      STD_ strstream ins((char *)conf_content, sizeof(conf_content));
#endif

      cool::ini_parser ip;
      cool::config c(ip, ins);
      cool::config::const_iterator i;
      for (i = c.begin();  i != c.end();  ++i) {
          const char *key = i->first;
          assert(c.exists(key) == true);
          assert(key[0] == 'k');

          const char *value = c.get(key);
          assert(value != 0);
          assert(value[0] == 'v');
          assert(key[1] == value[1]);
      }
  }
  catch (const cool::exception &e) {
      STD_ cout << e << STD_ endl;
      return 1;
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
