/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>
#include <stdio.h>

#if defined(INCLUDE_MODULE)
#  include <cool/util.hpp>
#else
#  include <cool/util/convert.hpp>
#endif

#include iostream_header


template <typename T>
void
test_of (const char *correct_value, T real_value,
        const char *out_of_range_value)
{
  char buf[128];
  snprintf(buf, sizeof(buf), "%sx123", correct_value);

  // check for correct_value
  char *eptr;
  T value = cool::as<T>(buf, &eptr);
  assert(value == real_value);
  assert(*eptr == 'x');

  // check for format_error
  try {
      value = cool::as<T>("x123", 0);
      assert(!"format_error should be thrown!");
  }
  catch (cool::format_error &e) {}

  // check for limit_error
  try {
      value = cool::as<T>(out_of_range_value, 0);
      assert(!"limit_error should be thrown!");
  }
  catch (cool::limit_error &e) {}
}


int
main (int argc, char *argv[])
{
  try {
      const char *out_of_range_value = "99999999999999999999";

      test_of<char>             ("-123", -123, out_of_range_value);
      test_of<unsigned char>    ("123",  123,  out_of_range_value);

      test_of<short>            ("-123", -123, out_of_range_value);
      test_of<unsigned short>   ("123",  123,  out_of_range_value);

      test_of<int>              ("-123", -123, out_of_range_value);
      test_of<unsigned int>     ("123",  123,  out_of_range_value);

      test_of<long>             ("-123", -123, out_of_range_value);
      test_of<unsigned long>    ("123",  123,  out_of_range_value);
  }
  catch (const cool::exception &e) {
      STD_ cout << e << STD_ endl;
      return 1;
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
