/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>

#if defined(INCLUDE_MODULE)
#  include <cool/net.hpp>
#else
#  include <cool/net/inet_socket.hpp>
#endif
#include iostream_header


void
test_address ()
{

  try {
      cool::inet::address::service("non-existing-service");
      assert(!"cool::service_not_found should be thrown!");
  }
  catch (cool::service_not_found &e) {}

  cool::inet::address a(cool::inet_address::loopback, 21);
  assert(a.port() == cool::inet::address::service("ftp"));
  assert(a.family() == cool::socket::inet);
  assert(a.size() == sizeof(sockaddr_in));

  cool::inet::address b(cool::inet_address::loopback, 21);
  assert(a == b);
  b.port(cool::inet::address::service("http"));
  assert(a != b);
  assert(cool::as<cool::inet::address>("localhost:http", 0) == b);

  assert(a.host("*").host().ip() == cool::inet_address::any);
}
  
  
void
test_send_recv (cool::inet::socket &sender, cool::inet::socket &receiver)
{

  char to_send[] = "hello, world";
  const int bufsz = sizeof(to_send) - 1;
  char to_recv[bufsz + 1];

  assert(sender.wait(cool::socket::out).writable() == true);
  assert(sender.send(to_send, bufsz) == bufsz);
      
  assert(receiver.wait(cool::socket::in).readable() == true);
  assert(receiver.recv(to_recv, sizeof(to_recv)) == bufsz);

  assert(strncmp(to_send, to_recv, bufsz) == 0);
} 


int
main (int argc, char *argv[])
{
  try {
      test_address();

      cool::inet::address a = "localhost:8192";
      cool::inet::listener server(a);
      assert(server.wait(cool::time::time(0, 50000000)).readable() == false);

      cool::inet::socket client;
      assert(client.is_connected() == false);
      assert(client.connect(a) == true);
      assert(client.is_connected() == true);

      // check accepting
      assert(server.wait().readable() == true);
      cool::inet::socket channel = server.accept(a);
      assert(channel.is_connected() == true);
      assert(client.peer() == channel.local());
      assert(client.local() == channel.peer());

      // test sending and receiving (both ways)
      test_send_recv(client, channel);
      test_send_recv(channel, client);

      // check that after closing, other side becomes readable but
      // there is no data
      client.close();
      assert(client.is_connected() == false);
      assert(channel.wait().readable() == true);
      char buf[1];
      assert(channel.recv(buf, sizeof(buf)) == 0);

      // other endpoint does not detect this condition
      // assert(channel.is_connected() == false);

      assert(channel != -1);
      assert(server != -1);
      {
          cool::socket::scoped_close channel_guard(channel);
          cool::socket::scoped_close server_guard(server);
      }
      assert(channel == -1);
      assert(server == -1);
  }
  catch (const cool::os_error &e) {
      STD_ cout << e << STD_ endl;
      return 1;
  }
  catch (const cool::exception &e) {
      STD_ cout << e << STD_ endl;
      return 1;
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
