/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>

#if defined(INCLUDE_MODULE)
#  include <cool/thread.hpp>
#else
#  include <cool/thread/mutex.hpp>
#  include <cool/thread/recursive_mutex.hpp>
#  include <cool/thread/condition.hpp>
#endif
#include iostream_header


template <typename mutex_impl>
void
test_lock ()
{
  typedef mutex_impl mutex_type;
  typedef typename mutex_impl::scoped_lock lock_type;

  mutex_type mutex;
  cool::condition condition;

  {
      lock_type lock(mutex, false);
      assert((!lock) == true);
  }

  lock_type lock(mutex);
  assert((!lock) == false);

  assert(condition.wait(lock, cool::time::time(0, 100)) == false);
  assert((!lock) == false);

  lock.unlock();
  assert((!lock) == true);
  lock.lock();
  assert((!lock) == false);
}


template <typename mutex_impl>
void
test_try_lock ()
{
  typedef mutex_impl mutex_type;
  typedef typename mutex_impl::scoped_try_lock try_lock_type;

  mutex_type mutex;
  cool::condition condition;

  {
      try_lock_type lock(mutex);
      assert((!lock) == false);
  }
  {
      try_lock_type lock(mutex, false);
      assert((!lock) == true);
  }

  try_lock_type lock(mutex, true);
  assert((!lock) == false);

  assert(condition.wait(lock, cool::time::time(0, 100)) == false);
  assert((!lock) == false);

  lock.unlock();
  assert((!lock) == true);
  lock.lock();
  assert((!lock) == false);
  lock.unlock();
  assert((!lock) == true);
  assert(lock.try_lock() == true);
  assert((!lock) == false);
}


template <typename mutex_impl>
void
test_timed_lock ()
{
  typedef mutex_impl mutex_type;
  typedef typename mutex_impl::scoped_timed_lock timed_lock_type;

  mutex_type mutex;
  cool::condition condition;

  {
      timed_lock_type lock(mutex, cool::time::time(0, 100));
      assert((!lock) == false);
  }
  {
      timed_lock_type lock(mutex, false);
      assert((!lock) == true);
  }

  timed_lock_type lock(mutex, true);
  assert((!lock) == false);

  assert(condition.wait(lock, cool::time::time(0, 100)) == false);
  assert((!lock) == false);

  lock.unlock();
  assert((!lock) == true);
  lock.lock();
  assert((!lock) == false);
  lock.unlock();
  assert((!lock) == true);
  assert(lock.timed_lock(cool::time::time(0, 100)) == true);
  assert((!lock) == false);
}


template <typename mutex_impl>
void
test_recursive_lock ()
{
  typedef mutex_impl mutex_type;
  typedef typename mutex_impl::scoped_lock lock_type;

  mutex_type m;
  lock_type guard1(m);
  lock_type guard2(m);
}


int
main (int argc, char *argv[])
{
  try {
      // fast mutex
      test_lock<cool::mutex>();

      test_lock<cool::try_mutex>();
      test_try_lock<cool::try_mutex>();

      test_lock<cool::timed_mutex>();
      test_try_lock<cool::timed_mutex>();
      test_timed_lock<cool::timed_mutex>();

      // recursive mutex
      test_lock<cool::recursive_mutex>();
      test_recursive_lock<cool::recursive_mutex>();

      test_lock<cool::recursive_try_mutex>();
      test_try_lock<cool::recursive_try_mutex>();
      test_recursive_lock<cool::recursive_try_mutex>();

      test_lock<cool::recursive_timed_mutex>();
      test_try_lock<cool::recursive_timed_mutex>();
      test_timed_lock<cool::recursive_timed_mutex>();
      test_recursive_lock<cool::recursive_timed_mutex>();
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
