/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>

#if defined(INCLUDE_MODULE)
#  include <cool/thread.hpp>
#else
#  include <cool/thread/blocking_queue.hpp>
#  include <cool/thread/bounded_queue.hpp>
#  include <cool/thread/locked_queue.hpp>
#  include <cool/thread/recursive_mutex.hpp>
#  include <cool/thread/thread.hpp>
#endif
#include iostream_header


template <typename queue_type>
struct consumer: public cool::thread::handler
{
  queue_type &queue_;
  int count_;

  consumer (queue_type &q, int count): queue_(q), count_(count) { }
  virtual ~consumer () { }

  void operator() () {
      try {
          typename queue_type::value_type i;
          for (i = 0;  i < count_;  ++i) {
              // do-while is not-so-eloquent way to compensate locked_queue
              // timing when producer is slower and consumer throws
              // not_found_error
              do {
                  try {
                      assert(queue_.pop() == i);
                      break;
                  }
                  catch (cool::not_found_error &e) {
                      // give producer chance to generate next item
                      cool::thread::yield();
                  }
              } while (true);
          }
      }
      catch (cool::exception &e) {
          STD_ cout << "consumer: " << e << STD_ endl;
          assert(!"Shouldn't happen!");
      }
  }
};


template <typename queue_type>
void
test_of (queue_type &queue, int count)
{
  consumer<queue_type> c(queue, count);
  cool::thread t(&c);

  typename queue_type::value_type i;
  for (i = 0;  i < count;  ++i)
      queue.push(i);

  t.join();
  assert(queue.size() == 0);
}


int
main (int argc, char *argv[])
{
  try {
      {
          cool::blocking_queue<int> queue;
          test_of(queue, 10);
      }
      {
          cool::bounded_queue<int> queue(3);
          test_of(queue, 10);
      }
      {
          cool::locked_queue<int> queue;
          test_of(queue, 10);
      }
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
