/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>

#if defined(INCLUDE_MODULE)
#  include <cool/signal.hpp>
#  include <cool/process.hpp>
#  include <cool/time.hpp>
#else
#  include <cool/signal/signal.hpp>
#  include <cool/process/process.hpp>
#  include <cool/time/time.hpp>
#endif
#include iostream_header


cool::process process;
const cool::time::sec_type sleep_time = 0;

const cool::signal::signal_type sig1 = cool::signal::usr1;
const cool::signal::signal_type sig2 = cool::signal::usr2;

static int term_signal_count = 0;


struct signal_handler: public cool::signal::handler
{
  signal_handler (cool::signal::signal_type s): cool::signal::handler(s) {}

  void operator () (cool::signal::signal_type s) {
      event_time.now();
      switch (s) {
          case sig1:
              kill(process.id(), sig2);
              sleep(sleep_time);
              assert(cool::signal::is_pending(sig2) == true);
              return;

          case sig2:
              return;

          case cool::signal::term:
              term_signal_count++;
              return;

          default:
              assert(!"Some other signal reached!");
      }
  }

  cool::time::time event_time;
};


int
main (int argc, char *argv[])
{
#if defined(COOL_POSIX)

  try {
      signal_handler s1(sig1), s2(sig2);

      cool::signal::mask(sig1, sig2);
      kill(process.id(), sig1);
      assert((s2.event_time.sec - s1.event_time.sec) >= sleep_time);
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  { // set our TERM handler
      signal_handler term(cool::signal::term);

      // signal should receive later
      cool::signal::mask(cool::signal::term);
      kill(process.id(), cool::signal::term);
      assert(term_signal_count == 0);

      // signal should receive immediately
      cool::signal::unmask(cool::signal::term);
      assert(term_signal_count == 1);
      kill(process.id(), cool::signal::term);
      assert(term_signal_count == 2);
  } // Default TERM handler must be restored

  /* On some platforms (HPUX) 'make' aborts execution if SIGTERM catched :(
  kill(process.id(), cool::signal::term);
  assert(!"cool::signal::term didn't arrive!");
  */

  return 0;

#else

  STD_ cout << "not implemented" << STD_ endl;
  return 1;

#endif
}
