/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>
#include <stdio.h>

#if defined(INCLUDE_MODULE)
#  include <cool/net.hpp>
#else
#  include <cool/net/socket.hpp>
#endif
#include iostream_header


int
main (int argc, char *argv[])
{
  try {
      cool::socket s;

      // comparing socket directly with integer (actually handler) is allowed
      // because of operator socket_type() that returns OS socket descriptor
      assert(s == -1);
      assert((!s) == true);
      assert(s.open() != -1);
      assert((!s) == false);

      // check initial socket options
      assert(s.debug() == false);
      assert(s.reuseaddr() == false);
      assert(s.keepalive() == false);
      assert(s.dontroute() == false);
      assert(s.broadcast() == false);
      assert(s.oobinline() == false);
      assert(s.blocking() == true);

      /* Unfortunately, there is no way to check buffer sizes as OS takes
       * these as recommendations
      assert(s.sendbufsz(2048).sendbufsz() == 2048);
      assert(s.recvbufsz(2048).recvbufsz() == 2048); */

      // check setting socket options
      try {
          // this is actually privileged option, don't bother if fails
          assert(s.debug(true).debug() == true);
      } catch (cool::permission_error &e) { }
      assert(s.reuseaddr(true).reuseaddr() == true);
      assert(s.keepalive(true).keepalive() == true);
      assert(s.dontroute(true).dontroute() == true); 
      assert(s.broadcast(true).broadcast() == true);
      assert(s.oobinline(true).oobinline() == true);
      assert(s.blocking(false).blocking() == false);

      // check if there is OOB waiting
      assert(s.atmark() == false);

      // check for errors
      assert(s.pending_error() == 0);

      // check lingering options
      cool::socket::linger_type l = s.linger();
      assert(l.l_onoff == 0 && l.l_linger == 0);
      l.l_onoff = true; l.l_linger = 5;
      l = s.linger(l).linger();
      assert(l.l_onoff == 1 && l.l_linger == 5);

      // check wait function (even though socket is not connected, it must be
      // writable anyway)
      assert(s.wait().writable() == true);

      // Create copy by cloning
      cool::socket o(s.clone());
      assert(o != s);
      assert(s.close() == -1);
      assert((!s) == true);
      assert(o.close() == -1);
      assert(o == s);
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
