/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>
#include <stdio.h>

#if defined(INCLUDE_MODULE)
#  include <cool/net.hpp>
#else
#  include <cool/net/socketstream.hpp>
#  include <cool/net/inet_socket.hpp>
#  include <cool/net/local_socket.hpp>
#endif
#include iostream_header


template <typename family>
void
test_socketstream_of (const typename family::address &a)
{
  typedef typename cool::socketstream<family> socketstream;
  typedef typename socketstream::listener_type listener;

  // create server
  listener server(a);

  // create client
  socketstream client(a);
  assert(client.is_open() == true);

  // accept i.e. create socket channel
  socketstream channel(server);
  assert(channel.is_open() == true);

  // test I/O
  for (int i = 0;  i < 2;  ++i) {
      int x;

      // client -> channel
      client << i << STD_ endl;
      channel >> x;
      assert(x == i);

      // channel -> client
      char data[] = "hello";
      channel << data << STD_ endl;
      char buf[1024];
      client >> STD_ ws;
      assert(client.getline(buf, sizeof(buf)).gcount() == sizeof(data));
      assert(strncmp(data, buf, sizeof(data) - 1) == 0);

      // channel -> client (with parse error)
      channel << data << STD_ endl;
      client >> x;
      assert(client.fail() != false);
      client.clear();
  }

  client.close();
  assert(client.is_open() == false);
  channel.close();
  assert(channel.is_open() == false);
  server.close();
}


int
main (int argc, char *argv[])
{
  try {
      // test inet socket
      test_socketstream_of<cool::inet>("localhost:8192");

      // test local (i.e. unix) socket
      const char *local_socket_name = "./my_socket";
      remove(local_socket_name);
      test_socketstream_of<cool::local>(local_socket_name);
  }
  catch (const cool::os_error &e) {
      STD_ cout << e << STD_ endl;
      return 1;
  }
  catch (const cool::exception &e) {
      STD_ cout << e << STD_ endl;
      return 1;
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
