/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>

#if defined(INCLUDE_MODULE)
#  include <cool/thread.hpp>
#else
#  include <cool/thread/thread_list.hpp>
#  include <cool/thread/mutex.hpp>
#  include <cool/thread/condition.hpp>
#endif
#include iostream_header


struct watchdog:
    public cool::thread::handler,
    public cool::condition
{
  cool::thread &parent_;
  cool::mutex mutex_;
  cool::time::time max_;

  watchdog (cool::thread &parent, const cool::time::time &t)
      : parent_(parent), max_(t) {}

  virtual ~watchdog () {}

  void operator() () {
      // comparison tests
      cool::thread self;
      assert(parent_ != self);
      cool::thread another_self;
      assert(self == another_self);

      // start watching
      cool::mutex::scoped_lock guard(mutex_);
      assert(wait(guard, max_) == true);
  }
};


int data;
void
thread_func (void *)
{
  data = 1;
}


int
main (int argc, char *argv[])
{
  try {
      cool::thread_list threads;
      cool::thread self;

      // start watchdog that prevents this test to run too long (if some
      // programming error happened)
      watchdog w(self, cool::time::time(1));
      assert(*threads.create(&w) != self);

      // start thread that changes value (but value can be checked later, when
      // thread is joined)
      data = 0;
      threads.create(thread_func);

      // signal watchdog that no need to bark
      w.notify_one();
      threads.join_all();

      assert(data == 1);
  }
  catch (cool::os_error &e) {
      STD_ cout << e << STD_ endl;
      return 1;
  }
  catch (cool::exception &e) {
      STD_ cout << e << STD_ endl;
      return 1;
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
