/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>

#if defined(INCLUDE_MODULE)
#  include <cool/thread.hpp>
#else
#  include <cool/thread/tss.hpp>
#  include <cool/thread/thread_list.hpp>
#endif
#include iostream_header


cool::mutex check_mutex;
cool::mutex tss_mutex;
int tss_instances = 0;
int tss_total = 0;


struct tss_value_t
{
  tss_value_t () {
      cool::mutex::scoped_lock guard(tss_mutex);
      ++tss_instances;
      ++tss_total;
      value = 0;
  }

  ~tss_value_t () {
      cool::mutex::scoped_lock guard(tss_mutex);
      --tss_instances;
  }

  int value;
};

cool::thread_specific_ptr<tss_value_t> tss_value;


void
thread_func (void *)
{
  tss_value.reset(new tss_value_t());
  for (int i = 0;  i < 100;  ++i) {
      int &n = tss_value->value;
      assert(n == i);
      ++n;
  }
}


int
main (int argc, char *argv[])
{
  try {
      const int num_of_threads = 5;
      cool::thread_list threads;
      for (int i = 0;  i < num_of_threads;  ++i)
          threads.create(thread_func);
      threads.join_all();

      assert(tss_instances == 0);
      assert(tss_total == num_of_threads);
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
