/* Copyright (c) 2003-2005, Sven Suursoho
 * See LICENSE for details */

#undef NDEBUG
#include <assert.h>

#if defined(INCLUDE_MODULE)
#  include <cool/net.hpp>
#  include <cool/exception.hpp>
#else
#  include <cool/net/url.hpp>
#  include <cool/exception/exception.hpp>
#endif
#include iostream_header


int
main (int argc, char *argv[])
{
  char *data[] = {
      "g:h",            "g:h",
      "g",              "http://a/b/c/g",
      "./g",            "http://a/b/c/g",
      "g/",             "http://a/b/c/g/",
      "/g",             "http://a/g",
      "//g",            "http://g",
      "?y",             "http://a/b/c/d;p?y",
      "g?y",            "http://a/b/c/g?y",
      "g?y/./x",        "http://a/b/c/g?y/./x",
      "#s",             "http://a/b/c/d;p?q#s",
      "g#s",            "http://a/b/c/g#s",
      "g#s/./x",        "http://a/b/c/g#s/./x",
      "g?y#s",          "http://a/b/c/g?y#s",
      ";x",             "http://a/b/c/d;x",
      "g;x",            "http://a/b/c/g;x",
      "g;x?y#s",        "http://a/b/c/g;x?y#s",
      ".",              "http://a/b/c/",
      "./",             "http://a/b/c/",
      "..",             "http://a/b/",
      "../",            "http://a/b/",
      "../g",           "http://a/b/g",
      "../..",          "http://a/",
      "../../",         "http://a/",
      "../../g",        "http://a/g",
      "",               "http://a/b/c/d;p?q#f",
      "../../../g",     "http://a/../g",
      "../../../../g",  "http://a/../../g",
      "/./g",           "http://a/./g",
      "/../g",          "http://a/../g",
      "g.",             "http://a/b/c/g.",
      ".g",             "http://a/b/c/.g",
      "g..",            "http://a/b/c/g..",
      "..g",            "http://a/b/c/..g",
      "./../g",         "http://a/b/g",
      "./g/.",          "http://a/b/c/g/",
      "g/./h",          "http://a/b/c/g/h",
      "g/../h",         "http://a/b/c/h",
      "http:g",         "http:g",
      "http:",          "http:"
  };
  int test_count = sizeof(data)/sizeof(data[0]) >> 1;

  try {
      int i;
      char buf[32];
      char base_url[] = "http://a/b/c/d;p?q#f";
      cool::url base = base_url;

      assert(strcmp(base.c_str(buf, sizeof(buf)), base_url) == 0);
      assert(base.port() == -1);
      assert(base.default_port() == 80);

      for (i = 0;  i < test_count;  ++i) {
          cool::url u = data[i << 1];
          assert(strcmp(u.c_str(buf, sizeof(buf)), data[i << 1]) == 0);

          u.resolve(base).c_str(buf, sizeof(buf));
          assert(strcmp(data[(i << 1) + 1], buf) == 0);

          u = data[(i << 1) + 1];
          assert(strcmp(u.c_str(buf, sizeof(buf)), data[(i << 1) + 1]) == 0);
      }
  }
  catch (STD_ exception &e) {
      STD_ cout << e.what() << STD_ endl;
      return 1;
  }

  return 0;
}
