#include <common/protocol.hpp>
#include <netinet/in.h>


namespace common
{


void
protocol::recv_hello (pooler_status_type &status, std::string &name)
{
  // receive status
  status = (pooler_status_type)recv<uint8_t>();
  if (status == offline)
      // receive name
      name = recv<std::string>();
}


std::string
protocol::recv_statistics ()
{
  // send query
  send((uint8_t)cmd_get_info);
  flush();

  // receive result
  response_type r = (response_type)recv<uint8_t>();
  assert(r == rsp_ok);
  r = rsp_ok;   // to keep compiler happy

  return recv<std::string>();
}


protocol::response_type
protocol::set_status (pooler_status_type &status)
{
  send((uint8_t)cmd_set_status);
  send((uint8_t)status);
  flush();
  return (response_type)recv<uint8_t>();
}


void
protocol::send_query (const query_type &q)
{
  send((uint8_t)cmd_exec_query);
  send(q.cluster);
  send(q.conf_map);
  send(q.hash_nr);
  send(q.query);
  flush();
}


protocol::response_type
protocol::recv_result (int32_t &rows, int32_t &cols)
{
  response_type r = (response_type)recv<uint8_t>();
  if (r == rsp_ok) {
      rows = recv<int32_t>();
      cols = recv<int32_t>();
  }

  return r;
}


const char *
protocol::as_string (response_type r)
{
  switch (r) {
      case rsp_ok:            
          return "POOL_OK"; 

      case rsp_invalid:
          return "DB_ERROR";

      case rsp_enotimpl:      
          return "POOL_ENOTIMPL"; 

      case rsp_enores:        
          return "POOL_ENORES"; 

      case rsp_enoconn:       
          return "POOL_ENOCONN"; 

      case rsp_enocluster:
          return "POOL_ENOCLUSTER"; 

      case rsp_enopartition:
          return "POOL_ENOPARTITION"; 
  }

  // never reached, but keeps compiler happy
  return "POOL_INTERNAL"; 
}


}   // namespace common
