/**
 * \file db/error.hpp
 * \brief PostgreSQL types info
 */

#ifndef db_error_hpp_included
#define db_error_hpp_included


#include <common/protocol.hpp>
#include <db/plproxy.hpp>
#include <stdexcept>


namespace db
{


/** \brief Parent for all error classes */
class error: public std::exception
{
  public:

      enum error_type
      {
          generic               = 1
          , spi                 = 2
          , null_hash           = 3
          , no_mem              = 4
          , not_impl            = 5
          , no_trigger          = 6
          , set_context         = 7
          , sys_cache           = 8
          , any                 = 9
          , type                = 10
          , src_null            = 11
          , syntax              = 12
          , pool_connect        = 13
          , pool_config         = 14

          , pool_error          = 1000
          , pool_db             = pool_error + common::protocol::rsp_invalid
          , pool_not_impl       = pool_error + common::protocol::rsp_enotimpl
          , pool_no_res         = pool_error + common::protocol::rsp_enores
          , pool_no_conn        = pool_error + common::protocol::rsp_enoconn
          , pool_no_cluster     = pool_error + common::protocol::rsp_enocluster
          , pool_no_partition   = pool_error + common::protocol::rsp_enopartition
      };
};


class syntax_error: public error
{
  public:

      syntax_error (const std::string &m): msg(m) {}

      virtual ~syntax_error () throw() {}

      virtual const char *what () const throw() {
          return msg.c_str();
      }


  private:

      std::string msg;
};


}   // namespace db


#endif  // db_error_hpp_included
