/**
 * \file db/plproxy.hpp
 * \brief PL/Proxy language handler library header file
 */

#ifndef db_plproxy_hpp_included
#define db_plproxy_hpp_included


#include <common/proxy.hpp>


// inet_net_ntop, inet_net_pton have different declarations in arpa/inet.h &
// server/utils/builtins.h. By undef _BSD_SOURCE & _SVID_SOURCE we can exclude
// arpa/inet.h
//
// XXX Unfortunately I can't find right place for undef, so use directly
#undef __USE_MISC


extern "C"
{


// debugging stuff
#if !defined(NDEBUG)
#  define USE_ASSERT_CHECKING
#  define assert_enabled plproxy_debug
#endif
extern bool plproxy_debug;

/* postgresql headers */
#include <postgres.h>
#include <fmgr.h>


/** \brief Language handler routine */
Datum plproxy_call_handler (PG_FUNCTION_ARGS);

/** \brief Debugging print */
void dprint (const char *fmt, ...);

/** \brief Shared library initialization routine */
void init () /*__attribute__((constructor))*/;

/** \brief Shared library finalization routine */
void done () __attribute__((destructor));


#define STRINGIFY(s) STRINGIFY2(s)
#define STRINGIFY2(s) #s
#define LOCUS "(PLPROXY:" __FILE__ ":" STRINGIFY(__LINE__) ") "



/* Copied from plpgsql:
 *
 * This routine is a crock, and so is everyplace that calls it.  The problem
 * is that the compiled form of a plpgsql function is allocated permanently
 * (mostly via malloc()) and never released until backend exit.  Subsidiary
 * data structures such as fmgr info records therefore must live forever
 * as well.  A better implementation would store all this stuff in a per-
 * function memory context that could be reclaimed at need.  In the meantime,
 * fmgr_info_cxt must be called specifying TopMemoryContext so that whatever
 * it might allocate, and whatever the eventual function might allocate using
 * fn_mcxt, will live forever too.
 */
void perm_fmgr_info (Oid functionId, FmgrInfo *finfo);


}   // extern "C"


#include <db/profiler.hpp>


#endif  // db_plproxy_hpp_included
