/**
 * \file db/pool.hpp
 * \brief Connection pooler stuff
 */

#ifndef db_pool_hpp_included
#define db_pool_hpp_included


#include <db/plproxy.hpp>
#include <common/protocol.hpp>
#include <string>


namespace db
{

using namespace common;


class pooler_client
{
  public:

      pooler_client () {
          connect();
      }


      virtual ~pooler_client () {
          disconnect();
      }


      bool connect ();


      void disconnect ();


      void send_query (std::string &query
              , std::string &cluster
              , std::string &conf_map, uint16_t &hash_nr);


      protocol::response_type recv_result (int32_t &rows, int32_t &cols) {
          profiler::function f("pooler_client::recv_result");
          return proto.recv_result(rows, cols);
      }


      std::string pooler_client::recv_result_field () {
          profiler::function f("pooler_client::recv_result_field");
          return proto.recv<std::string>();
      }

      static net_space::socket daemon;
      static protocol proto;
};


}   // namespace db


extern "C" {
extern db::pooler_client *pool;
}


#endif  // db_pool_hpp_included
