#include <db/profiler.hpp>

#if !defined(NDEBUG)
#  include <iostream>
#endif


namespace db
{


#if !defined(NDEBUG)


const uint32_t tabsz = 2;
const char tabch = ' ';

bool profiler::profiler_on = false;
uint64_t profiler::duration = 0;
uint64_t profiler::calls = 0;
uint64_t profiler::nest_level = 0;
profiler::data_map profiler::data;


void
profiler::enter (const std::string &fn)
{
  if (!profiler_on)
      return;

  std::cerr << std::string(tabsz*nest_level, tabch) << fn << " {" << std::endl;
  nest_level++;
  calls++;

  // update this function's data
  data_map::iterator i = data.find(fn);
  if (i == data.end()) {
      // first time to call this function, create new data structure
      data_type d;
      std::pair<data_map::iterator, bool> r =
          data.insert(std::make_pair(fn, d));
      i = r.first;
  }
  i->second.calls++;
  i->second.last.now();
}


void
profiler::leave (const std::string &fn)
{
  if (!profiler_on)
      return;

  cool::time now;
  data_map::iterator i = data.find(fn);
  Assert(i != data.end());

  uint64_t d = (now.sec - i->second.last.sec)*cool::time::usec_per_sec;
  d += (now.nsec - i->second.last.nsec)/cool::time::nsec_per_usec;

  // update statistics
  i->second.duration += d;
  duration += i->second.duration;
  nest_level--;

  std::cerr << std::string(tabsz*nest_level, tabch)
      << "} " << d << " usec(s)" << std::endl;
}


void
profiler::dump (std::ostream &os)
{
  if (!profiler_on || data.empty())
      return;

  os << "Profiler (total: calls=" << calls
      << "x, duration=" << duration << " usec)\n  calls / dur / avg";

  data_map::const_iterator i;
  for (i = data.begin();  i != data.end();  ++i) {
      os << "\n -" << i->first << ": " << i->second.calls
          << " / " << i->second.duration << " / ";
      os.precision(2);
      os << std::fixed << (float)i->second.duration/i->second.calls;
  }

  os << std::endl;
}


#endif


}   // namespace db
