/**
 * \file db/profiler.hpp
 * \brief System tuning
 */

#ifndef db_profiler_hpp_included
#define db_profiler_hpp_included


#include <db/plproxy.hpp>
#include <string>
#if !defined(NDEBUG)
#  include <cool/time/time.hpp>
#  include <map>
#endif


namespace db
{


/** \brief Profiler class -- collect call data and timings */
class profiler
{
  public:

      /**
       * \brief Function entry class -- just for remember function name and
       * calling profiler::enter
       */
      class function
      {
        public:

            function (const std::string &fn) {
#if !defined(NDEBUG)
                name = fn;
                profiler::enter(fn);
#endif
            }


            ~function () {
#if !defined(NDEBUG)
                profiler::leave(name);
#endif
            }


#if !defined(NDEBUG)
            std::string name;
#endif
      };


      /** \brief Reset profiler data */
      static void reset ()
      {
#if !defined(NDEBUG)
          duration = calls = nest_level = 0;
          data.clear();
#endif
      }


      /** \brief Reset nesting level counter */
      static void cancel_call_chain () {
#if !defined(NDEBUG)
          nest_level = 0;
#endif
      }


      /** \brief Switch profiler on/off */
      static void active (bool on)
      {
#if !defined(NDEBUG)
          profiler_on = on;
#endif
      }

      /** \brief Start function with name \a fn */
      static void enter (const std::string &fn)
#if defined(NDEBUG)
      {
      }
#endif
      ;


      /** \brief Leave function with name \a fn */
      static void leave (const std::string &fn)
#if defined(NDEBUG)
      {
      }
#endif
      ;


      static void dump (std::ostream &os)
#if defined(NDEBUG)
      {
      }
#endif
      ;


  private:

#if !defined(NDEBUG)
      static bool profiler_on;
      static uint64_t calls;
      static uint64_t duration;
      static uint64_t nest_level;


      struct data_type
      {
          data_type (): last(), calls(0), duration(0) {}

          cool::time last;
          uint64_t calls;
          uint64_t duration;
      };


      typedef std::map<std::string, data_type> data_map;
      static data_map data;
#endif
};


}   // namespace db


#endif  // db_profiler_hpp_included
