#include <db/serial.hpp>


extern "C" {
#include <db/hide_cpp_keywords.hpp>
#include <executor/spi.h>
#include <db/show_cpp_keywords.hpp>
}   // extern "C"


namespace db
{


int64_t
serial::next (const std::string &q)
{
  profiler::function f("serial::next");

  // select nextval
  //dprint("nextval(%s)\n", q.c_str());
  int r = SPI_exec(q.c_str(), 1);
  if (r != SPI_OK_SELECT)
      elog(ERROR, LOCUS"ESEQUENCE: %s", SPI_result_code_string(r));
  //dprint("nextval_result_count=%d\n", SPI_processed);
  Assert(SPI_processed == 1);

  // get value
  SPITupleTable *res = SPI_tuptable;
  TupleDesc rowdesc = res->tupdesc;
  HeapTuple row = res->vals[0];
  bool isnull;
  Datum d = SPI_getbinval(row, rowdesc, 1, &isnull);
  Assert(isnull != true);
  int64_t result = DatumGetInt64(d);
  //dprint("nextval_value=%d\n", result);

  // release query and return
  SPI_freetuptable(res);
  return result;
}


}   // namespace db
