-- create test functions

create or replace
function test_bytea (v text, data bytea, retnull boolean)
    returns bytea
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return data;
end;
$$;


create or replace
function test_text (v text, retnull boolean)
    returns text
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_character_17 (v character(17), retnull boolean)
    returns character(17)
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_varchar_15 (v varchar(15), retnull boolean)
    returns varchar(15)
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_integer (v integer, retnull boolean)
    returns integer
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_bigint (v bigint, retnull boolean)
    returns bigint
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_decimal (v decimal(10,2), retnull boolean)
    returns decimal(10,2)
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_real (v real, retnull boolean)
    returns real
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_double (v double precision, retnull boolean)
    returns double precision
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_tuple (v test_tuple_type, retnull boolean)
    returns test_tuple_type
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_boolean (v boolean, retnull boolean)
    returns boolean
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_timestamp (v timestamp, retnull boolean)
    returns timestamp
    language plpgsql
as $$
begin
    perform assert(v is not null);

    if retnull then
        return null;
    end if;

    return v;
end;
$$;


create or replace
function test_setof_integer (num_of_items integer, value integer)
    returns setof integer
    language plpgsql
as $$
declare
    i integer;
begin
    perform assert(num_of_items is not null);

    for i in 1..num_of_items loop
        return next value;
    end loop;

    return;
end;
$$;


create or replace
function test_setof_tuple (num_of_items integer, value test_tuple_type)
    returns setof test_tuple_type
    language plpgsql
as $$
declare
    i integer;
begin
    perform assert(num_of_items is not null);

    for i in 1..num_of_items loop
        return next value;
    end loop;

    return;
end;
$$;


create or replace
function test_setof_row (num_of_rows integer)
    returns setof pg_cast
    language plpgsql
as $$
declare
    r pg_cast%ROWTYPE;
begin
    /* return num_of_rows from pg_views */
    for r in select * from pg_cast limit num_of_rows loop
        return next r;
    end loop;

    return;
end;
$$;
