/**
 * \file db/typeinfo.hpp
 * \brief PostgreSQL types info
 */

#ifndef db_typeinfo_hpp_included
#define db_typeinfo_hpp_included


#include <db/plproxy.hpp>


namespace db
{


/** \brief Class representing PostgreSQL type */
class typeinfo
{
  public:

      /** \brief Create type info structure */
      typeinfo (): kind(invalid) { }


      virtual ~typeinfo () throw() {}


      /** \brief Initialize type structure */
      Oid init (Oid typid, bool in);


      /**
       * \brief Get \a arg as C nul-terminated char array.
       *
       * Converts to string, and sets \a as_string to point it's memory area.
       * Buffer is allocated using palloc() i.e. is freed at end of
       * transaction or can be freed early using pfree().
       *
       * If \a need_quotes == true then string is escaped before returning.
       */
      char *as_string (Datum arg);


      /** \brief Return value of \a str as datum */
      Datum as_datum (const char *str) {
          profiler::function f("typeinfo::as_datum");
          return FunctionCall3(&io, CStringGetDatum(str),
                  ObjectIdGetDatum(io_param), Int32GetDatum(-1));
      }


      /** \brief PostgreSQL type Oid */
      Oid oid;


      /** \brief return by value */
      bool by_value;


      /** \brief type length */
      int type_length;


      /** \brief True, if type must have quoted and escaped in SQL queries */
      bool need_quotes;


      /** \brief I/O parameter */
      Oid io_param;


      /** \brief Lookup info for I/O function */
      FmgrInfo io;


      /** \brief Type kind */
      enum kind_type {
          invalid,          /**< Type info is uninitialized */
          scalar,           /**< Scalar type or domain */
          row,              /**< Composite */
      };
      kind_type kind;
};


}   // namespace db


#endif  // db_typeinfo_hpp_included
