#ifndef pgclient_error_hpp_included
#define pgclient_error_hpp_included


/**
 * \file pgclient/error.hpp
 * \brief Database error class
 */


#include <pgclient/pg.hpp>
#include <cool/exception/exception.hpp>


namespace pgclient
{


class connection;


class error: public cool::exception
{
  public:

      /** \brief Construct exception object with message \a msg */
      error (const char *msg, bool strip_newline = false) {
          what(msg, strip_newline);
      }


      /** \brief Construct exception object with message got from connection */
      error (connection &c);


      /** \brief Returns message describing error reason */
      virtual const char *what () const throw() {
          return message_;
      }

      /** \brief Set new error message for exception */
      void what (const char *msg, bool strip_newline = false);


  private:

      enum { message_bufsz = 1024 };
      char message_[message_bufsz];
};


}   // namespace pgclient


#endif  // pgclient_error_hpp_included
