#include <pool/config.hpp>
#include <pool/error.hpp>
#include <pool/logger.hpp>
#include <pgclient/connection.hpp>


namespace pgcpool
{


std::string config::app;
std::string config::self;

unsigned short config::log_level = 3;
std::string config::log_file = "stdout";

bool config::daemon = false;

common::net_space::address config::listen = common::protocol::rendevouz;
unsigned short config::max_clients = 16;
size_t config::max_request = 8;

std::string config::db_connect = "";

int config::reconnect_interval = 5;
int config::lifetime = 30;

std::string config::statistics_prefix = ".info-";
time_t config::statistics_interval = 0;


static inline void
wait_until_ready (pgclient::connection &c)
{
  while (!c.wait(1).ready())
      ;
}


bool
config::load ()
{
  // query runtime configuration from database
  pgclient::connection proxydb(db_connect.c_str());
  wait_until_ready(proxydb);
  std::string q = "SELECT * FROM plproxy.get_proxy_config('" + self + "')";
  pgclient::result r(q.c_str());
  wait_until_ready(proxydb.exec(&r));

  // fetch config
  for (int i = 0;  i < r.size();  ++i) {
      std::string key(r[i][0].c_str());
      if (key == "daemon")
          daemon = cool::as<bool>(r[i][1].c_str());
      else if (key == "lifetime") {
          lifetime = cool::as<int>(r[i][1].c_str());
          if (lifetime && lifetime < 10) {
              warning() << "config: 'lifetime' can't be <10min";
              lifetime = 10;
          }
      }
      else if (key == "listen")
          listen = r[i][1].c_str();
      else if (key == "log-file")
          log_file = r[i][1].c_str();
      else if (key == "log-level")
          log_level = cool::as<int>(r[i][1].c_str());
      else if (key == "max-clients")
          max_clients = cool::as<int>(r[i][1].c_str());
      else if (key == "max-request")
          max_request = cool::as<int>(r[i][1].c_str());
      else if (key == "reconnect-interval") {
          reconnect_interval = cool::as<int>(r[i][1].c_str());
          if (reconnect_interval < 1) {
              warning() << "config: 'reconnect-interval' can't be <1sec";
              reconnect_interval = 1;
          }
      }
      else if (key == "statistics-interval")
          statistics_interval = cool::as<int>(r[i][1].c_str());
      else if (key == "statistics-prefix")
          statistics_prefix = r[i][1].c_str();
  }

  show();
  return true;
}


bool
config::reload ()
{
  /*
  namespace po = boost::program_options;
  namespace fs = boost::filesystem;

  std::string config_file = app + ".conf";

  // check, if we still have access to configuration file
  if (!fs::exists(config_file.c_str())) {
      warning() << "config: No file: " << config_file;
      return false;
  }

  // currently, log-* are only reloadable configuration variables
  // unfortunately, parse_config_line does not support unknown options
  // so, we have to add all existing variables with dummy storage :(
  po::options_description cfo("");
  cfo.add_options()
      ("log-file",
           po::value<std::string>(&log_file)->default_value(log_file),
           "log file")
      ("log-level",
           po::value<unsigned short>(&log_level)->default_value(log_level),
           "log level")

      // dummy variables
      ("self", po::value<std::string>(), "")
      ("statistics-prefix", po::value<std::string>(), "")
      ("daemon", po::value<bool>(), "")
      ("listen", po::value<std::string>(), "")
      ("max-clients", po::value<unsigned short>(), "")
      ("max-request", po::value<size_t>(), "")
      ("db-connect", po::value<size_t>(), "")
      ("reconnect-interval", po::value<int>(), "")
      ("lifetime", po::value<int>(), "")
      ("statistics-interval", po::value<time_t>(), "")
  ;

  po::variables_map vars;
  std::ifstream cf(config_file.c_str());
  po::store(po::parse_config_file(cf, cfo), vars);
  po::notify(vars);
  */

  return true;
}


void
config::show ()
{
  debug() << app << ':'
      << "\n\tself: " << self
      << "\n\tlog_file: " << log_file
      << "\n\tlog_level: " << log_level
      << "\n\tstatistics_prefix: " << statistics_prefix
      << "\n\tdaemon: " << std::boolalpha << daemon
      << "\n\tlisten: " << listen << " (max " << max_clients << " client(s))"
      << "\n\tmax_request: " << max_request << "kb"
      << "\n\tdb_connect: " << db_connect
      << "\n\treconnect_interval: " << reconnect_interval << "sec"
      << "\n\tlifetime: " << lifetime << "min"
      << "\n\tstatistics_interval: " << statistics_interval << "sec"
      ;

  max_request *= 1024;          // kb -> bytes
  lifetime *= 60;               // min -> sec
}


}   // namespace pgcpool
