#ifndef pool_config_hpp_included
#define pool_config_hpp_included

/**
 * \file pool/config.hpp
 * \brief PG connection pooler daemon configuration
 */


#include <pool/pgcpoold.hpp>
#include <common/protocol.hpp>
#include <string>


namespace pgcpool
{


class config
{
  public:

      /** \brief Load configuration file and parse command line options */
      static bool load ();

      /** \brief Reload configuration file */
      static bool reload ();

      /** \brief Application name */
      static std::string app;

      /** \brief Log file level */
      static unsigned short log_level;

      /** \brief Log file */
      static std::string log_file;

      /**
       * \brief Flag showing if we have to give up console and run in
       * bacground
       */
      static bool daemon;

      /** \brief Partition mapping value for pl/proxy */
      static std::string self;

      /** \brief Server rendevouz-point */
      static common::net_space::address listen;

      /** \brief Number of maximum active clients */
      static unsigned short max_clients;

      /** \brief Maximum request size */
      static size_t max_request;

      /** \brief Connect string for proxydb */
      static std::string db_connect;

      /**
       * \brief Interval (seconds) for attempting to repair broken partition
       * connections
       */
      static int reconnect_interval;

      /**
       * \brief Lifetime in seconds for connection to partition (after which
       * connection is re-established).
       * In config it is specified in minutes and converted to seconds by
       * load(). If 0, connection lasts forever.
       */
      static int lifetime;

      /** \brief Statistics filename prefix */
      static std::string statistics_prefix;

      /**
       * \brief Number of seconds between showing statistics for pool.
       *
       * Set 0 to disable.
       */
      static time_t statistics_interval;


  private:

      // show configuration content
      static void show ();
};


}   // namespace pgcpool


#endif  // pool_config_hpp_included
