#ifndef pool_error_hpp_included
#define pool_error_hpp_included

/**
 * \file pool/error.hpp
 * \brief PG connection pooler errors
 */


#include <pool/pgcpoold.hpp>
#include <cool/exception.hpp>


namespace pgcpool
{


/** \brief Parent class for pool errors */
class exception: public cool::exception
{
  public:

      /** \breif Additional information */
      const char *info;

      exception (const char *info): info(info) {}

      virtual ~exception () throw() {}

      virtual const char *what () const throw() {
          return "exception";
      }


      /** \brief Global exception handler. Called only if local is not found */
      static void global_handler() throw();
};


/** \brief Macro to simplify creation of new exception classes */
#define ERROR_CLASS(name,parent) \
    class name: public parent { \
      public: \
          name (const char *info): parent(info) {} \
          virtual ~name () throw() {} \
          virtual const char *what () const throw() { \
              return #name; \
          } \
    }      


ERROR_CLASS(integrity_error, exception);


}   // namespace pgcpool


#endif  // pool_error_hpp_included
