#include <pool/pgcpoold.hpp>
#include <pool/config.hpp>
#include <pool/error.hpp>
#include <pool/listener.hpp>
#include <pool/logger.hpp>
#include <pool/partitions.hpp>
#include <pool/signal.hpp>

#include <cool/process/process.hpp>
#include <boost/filesystem/path.hpp>


using namespace pgcpool;


static void close_io ();


int
main (int argc, char *argv[])
{
  int return_value = EXIT_SUCCESS;

  try {
      namespace fs = boost::filesystem;
      config::self = config::app = fs::path(argv[0]).leaf();

      // load config
      if (argc > 1)
          config::db_connect = argv[1];
      if (!config::load())
          return EXIT_FAILURE;

      log_stream::configure(config::log_file
          , static_cast<log_level::value>(config::log_level));

// starting

      // setup signal handlers
      signal_handler intr(cool::signal::intr);
      signal_handler quit(cool::signal::quit);
      signal_handler term(cool::signal::term);
      signal_handler hup(cool::signal::hup);
      cool::signal::mask(cool::signal::pipe);

      // create listening point
      listener server(config::listen, config::max_clients);

      // go into background
      if (config::daemon) {
          debug() << "daemonize";
          cool::process p;
          p.detach();
          close_io();
      }

      // start partition handlers
      if (!partitions::load_conf(config::db_connect, config::self))
          signal_handler::shutdown = true;
      partitions::start_handlers();

// ready
      info() << "ready";
      while (!signal_handler::shutdown) {
          try {
              server.wait_and_dispatch();
          }
          catch (const cool::interrupted &/*unused*/) {}
          catch (...) {
              warning() << "main()";
              throw;
          }
      }
      info() << "stop";

// stopping
      partitions::stop_handlers();
  }
  catch (const cool::in_use_error &e) {
      std::cerr << config::listen << " in use. Are we already running?"
          << std::endl;
      return_value = EXIT_FAILURE;
  }
  catch (const pgclient::error &e) {
      std::cerr << e << std::endl;
      return_value = EXIT_FAILURE;
  }
  catch (...) {
      exception::global_handler();
      return_value = EXIT_FAILURE;
  }

  return return_value;
}


static void
close_io ()
{
  close(STDIN_FILENO);
  close(STDOUT_FILENO);
  close(STDERR_FILENO);
  open("/dev/null", O_RDONLY);
  dup2(STDOUT_FILENO, STDERR_FILENO);
}
