#include "test.hpp"
#include <pgclient/connection.hpp>
#include <cool/net/exception.hpp>
#include <iostream>


using namespace std;


inline void
wait (pgclient::connection &c)
{
  while (!c.wait(1).ready())
      ;
}


int
main (void)
{
  try {
      pgclient::connection conn("invalid-connection-string");
      fail("Shouldn't get here: used invalid connection string");
  }
  catch (const pgclient::error &e) { /*expected*/ }

  try {
      pgclient::connection conn("");

      // wait until connected
      wait(conn);

      pgclient::result r("");
      assert(r.empty() == true);
      assert(r.size() == 0);

      // send invalid query
      r.sql("invalid-sql-statement");
      try {
          wait(conn.exec(&r));
          fail("previous query had to be failed");
      }
      catch (const pgclient::error &e) { /*expected*/ }

      // send another, correct one with known result for checking
      r.sql("SELECT typname, typlen, null AS dummy "
              "FROM pg_type "
              "WHERE typname='int4'");
      while (true) {
          try {
              wait(conn.exec(&r));
              break;
          }
          catch (const cool::connection_error &e) {
              // safe-guard against broken connection
              wait(conn.reset());
          }
      }

      assert(r.size() == 1);
      assert(r.empty() == false);
      assert(r.fields() == 3);
      assert(r[0].size() == r.fields());

      assert(strcmp(r.field_name(0), "typname") == 0);
      assert(strcmp(r.field_name(1), "typlen") == 0);
      assert(strcmp(r.field_name(2), "dummy") == 0);

      assert(r[0].field_number("typname") == 0);
      assert(r[0].field_number("typlen") == 1);
      assert(r[0].field_number("dummy") == 2);

      assert(strcmp(r[0]["typname"].c_str(), "int4") == 0);
      assert(r[0]["typname"].is_null() == false);
      assert(cool::as<int>(r[0]["typlen"].c_str()) == sizeof(int));
      assert(r[0]["typlen"].is_null() == false);
      assert(r[0]["dummy"].is_null() == true);

      try {
          (void)r.at(1)["typname"].c_str();
          fail("got value for invalid row");
      }
      catch (const cool::limit_error &e) { /*expected*/ }

      try {
          (void)r[0]["invalid-field"].c_str();
          fail("got value for invalid field");
      }
      catch (const cool::not_found_error &e) { /*expected*/ }
  }
  catch (const std::exception &e) {
      fail(e.what());
  }
  ok();
}
