#include "test.hpp"
#include <pgclient/pool.hpp>
#include <iostream>

  
using namespace std;


class myres: public pgclient::pool::result
{
  public:
  
      myres (int expected): pgclient::pool::result(0), expected_(expected) { }


      void on_error (error_type code, const pgclient::error &e) throw() {
          assert((code == no_connection && expected_ == -1) ||
                  (code == db_error && expected_ == -2));
          failure_count++;
      }

      void on_ready () throw() {
          assert(expected_ == cool::as<int>((*this)[0]["value"].c_str()));
          success_count++;
      }

      int expected_;
      static size_t failure_count;
      static size_t success_count;
};


size_t myres::failure_count = 0;
size_t myres::success_count = 0;


int
main (void)
{
  try {
      // create pool and wait until connected
      pgclient::pool pool("", 3, 3, 10, 10);
      while (pool.idle() != pool.size())
          pool.wait();

      for (int i = 0;  i < 2;  i++) {
          // register queries for asynchronous execution
          myres r0(0);     pool.exec(&(r0.sql("select 0 as value")));
          myres r1(1);     pool.exec(&(r1.sql("select 1 as value")));
          myres r2(2);     pool.exec(&(r2.sql("select 2 as value")));
          myres i(-1);     pool.exec(&(i.sql("select 99 as value")));

          // wait until all executed
          while (pool.idle() != pool.size())
              pool.wait();

          // register invalid query and wait result
          i.expected_ = -2;
          pool.exec(&(i.sql("invalid-sql-statement")));
          while (pool.idle() != pool.size())
              pool.wait();
      }

      // check result counts
      assert(myres::success_count == 6);
      assert(myres::failure_count == 4);
  }
  catch (const std::exception &e) {
      fail(e.what());
  }

  ok();
}
