CREATE OR REPLACE FUNCTION plproxy.alter_cluster_copy_conf(
    i_cluster_name text, 
    i_conf_name text) 
RETURNS text AS $$
-- used by:: admins to start working on new version of the cluster configuration
--   creates copy of existing configuration (all maps are copied also)
-- exceptions:
--   400 Cluster with given name NOT found
--   400 Configuration NOT found
-- returns:: 200 Ok
-- example:: select * from plproxy.copy_conf('eventdb','eventdb_V1');  
DECLARE
    v_id_cluster int;
    r_conf record;
    r_cluster record;
    v_id_cluster_conf int;
    v_conf_name text;
    v_version int;
BEGIN	
    select id_cluster into v_id_cluster from plproxy.db_cluster where cluster_name = lower(i_cluster_name);
    if v_id_cluster is null then 
        raise exception '400 Cluster with given name="%" not found', i_cluster_name;
    end if;

    for r_conf in 
        select * from plproxy.db_cluster_conf where conf_name = i_conf_name and key_cluster = v_id_cluster
    loop
        -- create new configuration
        v_id_cluster_conf = nextval('plproxy.proxy_seq');
        select coalesce( max(version) , 0 ) + 1 into v_version from plproxy.db_cluster_conf where key_cluster = v_id_cluster;
        v_conf_name = i_cluster_name || '_v' || v_version::text;
        insert into plproxy.db_cluster_conf(id_cluster_conf, key_cluster, conf_name, version)
        values(v_id_cluster_conf, r_conf.key_cluster, v_conf_name, v_version);
    
        -- copy database partitioning information
        insert into plproxy.db_conf_partition(key_cluster_conf, key_partition, hash_nr)
        select v_id_cluster_conf, key_partition, hash_nr
        from plproxy.db_conf_partition
        where key_cluster_conf = r_conf.id_cluster_conf;
    end loop;
    
    return '200 Ok';
END;
$$ LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;
