CREATE OR REPLACE FUNCTION plproxy.alter_cluster_drop_conf(
    i_cluster_name text ,
    i_conf_name text ,
    i_conf_map text ) 
RETURNS text AS $$
-- used by:: admin to remove all data for given cluster configuration
-- br:: active cluster configurations can not be deleted just for safety reasons
-- br:: proxy status must be OFFLINE
-- exceptions:
-- 400 Cluster with given name not found
-- 400 Active cluster configurations must NOT be deleted
-- returns:: 200 Ok
-- example:: select * from plproxy.alter_cluster_drop_conf('eventdb_v1');  
DECLARE
    r_cluster record;
    r_conf record;
BEGIN	
    r_cluster = plproxy.fetch_cluster_by_name(i_cluster_name);
    r_conf = plproxy.fetch_cluster_conf_by_name(i_conf_name, i_conf_map);
    if exists (
        select 1 from plproxy.db_conf 
	where cluster_name = lower(i_cluster_name)
	and conf_name = lower(i_conf_name) )
    then
        raise exception '400 Active configuration can NOT be modified!';
    end if;

    -- delete mappings
    delete from plproxy.db_conf_partition where key_cluster_conf = r_conf.id_cluster_conf;
    -- delete configuration
    delete from plproxy.db_cluster_conf where id_cluster_conf = r_conf.id_cluster_conf;

    return '200 ok';
END;
$$ LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;
