CREATE OR REPLACE FUNCTION plproxy.create_cluster(
    i_cluster_name text,
    i_cluster_description text,
    i_db_count int)
    RETURNS text AS $$
-- used by:: admins to create new clusters
--   creates configuration records for new database cluster set
-- exceptions:
-- returns:: 200 Ok
-- example:: select * from plproxy.create_cluster('userdb','Skype user database',3);  
DECLARE
    C_BITS constant int = 8;
    C_PARTITIONS constant int = 256;
    
    v_id_cluster int;
    v_id_cluster_conf int;
    v_id_partition int;
    r record;
BEGIN	
    -- cluster
    v_id_cluster = nextval('plproxy.proxy_seq');
    insert into plproxy.db_cluster(id_cluster, cluster_name, cluster_descr, cluster_bits, cluster_partitions)
    values(v_id_cluster, lower(i_cluster_name), i_cluster_description, C_BITS, C_PARTITIONS);

    -- cluster conf
    v_id_cluster_conf = nextval('plproxy.proxy_seq');
    insert into plproxy.db_cluster_conf(id_cluster_conf, key_cluster, conf_name, version)
    values(v_id_cluster_conf, v_id_cluster, lower(i_cluster_name) || '_v1' , 1);

    for i in 1 .. i_db_count loop
        -- database
        v_id_partition = nextval('plproxy.proxy_seq');
        insert into plproxy.db_partition(id_partition, key_cluster, partition_name)
        values(v_id_partition, v_id_cluster, plproxy.fetch_free_partition_name(i_cluster_name));
        
        -- database mapping
        for r in select generate_series(i,C_PARTITIONS,i_db_count) as partnr loop
            insert into plproxy.db_conf_partition(key_cluster_conf, key_partition, hash_nr)
            values(v_id_cluster_conf, v_id_partition, r.partnr-1);
        end loop;
    end loop;
    return '200 Ok';
END;
$$ LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;
