CREATE OR REPLACE FUNCTION plproxy.create_proxy(
    i_proxy_name text ,
    i_proxy_descr text ,
    i_cluster_list text[] ,
    i_conn_count_min int ,
    i_conn_count_max int )
RETURNS text AS $$
-- USED BY: admins when new proxy database and pool server is created
--   creates configuration records for new proxy server
-- RETURNS: 200 Ok
-- EXAMPLE: select * from plproxy.create_proxy('eventfe','EventDB and ContactDB',ARRAY['eventdb','contactdb','userdb'],1,100);  
DECLARE
    v_id_proxy int;
    r_cluster record;
    r_part record;
BEGIN	
    select id_proxy from plproxy.db_proxy where proxy_name = i_proxy_name into v_id_proxy;
    if not found then
        -- create proxy record
        v_id_proxy = nextval('plproxy.proxy_seq');
        insert into plproxy.db_proxy(id_proxy, proxy_name, proxy_descr)
        values(v_id_proxy, i_proxy_name, i_proxy_descr);
    end if;

    -- for each cluster in array add configuration records to connection pool
    for i in array_lower(i_cluster_list,1) .. array_upper(i_cluster_list,1) loop
        r_cluster = plproxy.fetch_cluster_by_name(i_cluster_list[i]);  
        for r_part in
            select id_partition from plproxy.db_partition where key_cluster = r_cluster.id_cluster
        loop
            insert into plproxy.db_proxy_connection(key_proxy, key_partition, conn_count_min, conn_count_max)
            values (v_id_proxy, r_part.id_partition, i_conn_count_min, i_conn_count_max);
        end loop;
    end loop;
    
    perform plproxy.alter_proxy(i_proxy_name, 'log-file', '/tmp/pgcpoold.log');
    perform plproxy.alter_proxy(i_proxy_name, 'log-level', '3');
    perform plproxy.alter_proxy(i_proxy_name, 'daemon', 'false');
    perform plproxy.alter_proxy(i_proxy_name, 'listen', '/tmp/.plproxy.sock');
    perform plproxy.alter_proxy(i_proxy_name, 'max-clients', '500');
    perform plproxy.alter_proxy(i_proxy_name, 'max-request', '128');
    perform plproxy.alter_proxy(i_proxy_name, 'reconnect-interval', '1');
    perform plproxy.alter_proxy(i_proxy_name, 'lifetime', '30');
    perform plproxy.alter_proxy(i_proxy_name, 'statistics-interval', '60');
    perform plproxy.alter_proxy(i_proxy_name, 'statistics-prefix', '/tmp/');

    return '200 Ok';
END;
$$ LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;
