CREATE OR REPLACE FUNCTION plproxy.deactivate_cluster_conf(
    i_cluster_name text , 
    i_conf_name text ) 
RETURNS text AS $$
-- used by:: admins when when cluster goes out of use
--   deactivates configuration so it can be deleted
-- exceptions:
--   400 Cluster with given name not found
--   400 Active cluster configurations must NOT be deleted
-- returns:: 200 Ok
-- example:: select * from plproxy.deactivate_cluster_conf(13, 4);  
DECLARE
    r_cluster record;
    r_conf record;
BEGIN	
    select * into r_cluster from plproxy.fetch_cluster_by_name(i_cluster_name);
    
    if not exists(
        select * from plproxy.db_cluster_conf 
        where conf_name = lower(i_conf_name) 
        and  key_cluster = r_cluster.id_cluster)
    then
        raise exception '400 Configuration with name="%" not found for cluster="%"!', i_conf_name, i_cluster_name;
    end if;

    delete from plproxy.db_conf where cluster_name = lower(i_cluster_name) and conf_name = lower(i_conf_name);

    return '200 Ok';
END;
$$ LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;
