CREATE OR REPLACE FUNCTION plproxy.drop_cluster(
    i_cluster_name text ) 
RETURNS text AS $$
-- used by:: admins to remove out of use clusters
--   removes all data for given cluster
-- exceptions:
--   400 Cluster with given name not found
--   400 Active cluster configurations must NOT be deleted
-- returns:: 200 Ok
-- example:: select * from plproxy.delete_cluster('userdb');  
DECLARE
    r_cluster record;
    r_conf record;
BEGIN	
    r_cluster = plproxy.fetch_cluster_by_name(i_cluster_name);
    if exists(select 1 from plproxy.db_conf where cluster_name = lower(i_cluster_name)) then
        raise exception '400 Cluster with active configuration should not be dropped!';
    end if;

    -- remove cluster configurations
    for r_conf in 
        select id_cluster_conf 
        from plproxy.db_cluster_conf 
        where key_cluster = r_cluster.id_cluster 
    loop
        -- delete mappings
        delete from plproxy.db_conf_partition where key_cluster_conf = r_conf.id_cluster_conf;
        -- delete configuration
        delete from plproxy.db_cluster_conf where id_cluster_conf = r_conf.id_cluster_conf;
    end loop;

    -- delete cluster databases
    delete from plproxy.db_proxy_connection where key_partition in (
        select id_partition from plproxy.db_partition where key_cluster = r_cluster.id_cluster);

    -- delete cluster databases
    delete from plproxy.db_partition where key_cluster = r_cluster.id_cluster;

    -- cluster
    delete from plproxy.db_cluster where id_cluster = r_cluster.id_cluster;

    return '200 Ok';
END;
$$ LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;
