CREATE OR REPLACE FUNCTION plproxy.delete_proxy(
    i_proxy_name text)
    RETURNS text AS $$
-- used by:: admin to deletes proxy configuration records when proxy is no longer needed
-- br:: also connection records must be deleted
-- br:: proxy status must be OFFLINE
-- returns:: 200 Ok
-- example:: select * from plproxy.delete_proxy('central_p1');  
DECLARE
    r_proxy record;
BEGIN	
    -- get proxy record
    r_proxy = plproxy.fetch_proxy_by_name(i_proxy_name);

    if exists (select 1 from plproxy.db_proxy_connection where key_proxy=r_proxy.id_proxy) then
        raise exception '400 Only proxies without connections can be deleted.';
    end if;

    delete from plproxy.db_proxy_connection where key_proxy = r_proxy.id_proxy;

    delete from plproxy.db_proxy_config where key_proxy = r_proxy.id_proxy;

    update plproxy.db_server set key_proxy = null where key_proxy = r_proxy.id_proxy;

    delete from plproxy.db_proxy where id_proxy = r_proxy.id_proxy;
    
    return '200 Ok';
END;
$$ LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;
