DROP TYPE plproxy.ret_get_active_confs CASCADE;

CREATE TYPE plproxy.ret_get_active_confs AS (
  cluster_name text ,
  conf_name text );

CREATE OR REPLACE FUNCTION plproxy.get_active_confs(
    i_proxy_name text) 
RETURNS SETOF plproxy.ret_get_active_confs AS $$
-- used by:: proxy pool to initialize internal list of served clusters
--   needed when pool is started or when configuration is changed or reloaded
--   result is used for fetching internal maps for each cluster
-- br:: only clusters served by given proxy are returned
-- returns:: set of records
--   cluster_name text - language knows cluster
--   conf_name text - just for reference
-- example::
--   select * from plproxy.get_active_confs('central_p1');  
select cluster_name, conf_name from plproxy.db_conf
where cluster_name in (
  select cluster_name from plproxy.db_cluster where id_cluster in (
    select key_cluster from plproxy.db_partition where id_partition in (
      select key_partition from plproxy.db_proxy_connection where key_proxy in (
        select id_proxy from plproxy.db_proxy where proxy_name = lower($1) ))))
order by cluster_name;
$$
  LANGUAGE 'sql' VOLATILE SECURITY DEFINER;
