DROP TYPE plproxy.ret_get_cluster_map CASCADE;

CREATE TYPE plproxy.ret_get_cluster_map AS (
  conf_map text ,
  hash_nr int2 ,
  key_partition int4 );

CREATE OR REPLACE FUNCTION plproxy.get_cluster_maps(
    i_cluster_name text) 
RETURNS SETOF plproxy.ret_get_cluster_map AS $$
-- used by:: proxy pool to initialize internal maps for given cluster
--   needed when pool is started or when configuration is changed or reloaded
--   result is used for mapping conf_map and hash_nr received from hash function
--   to one partition and then result from get_cluster_partition is used to
--   get actual connect string
-- br:: maps only for active configuration are loaded
-- returns:: set of records
--   conf_map text -- map_code if several maps used otherwise '-'
--   hash_nr int2 - hash number calculated from username or sth else
--   key_partition int4 - partiton that holds this data
-- example::
--   select * from plproxy.get_cluster_maps('eventdb');  
select conf.conf_map, map.hash_nr, map.key_partition
  from plproxy.db_conf_partition map
       inner join plproxy.db_cluster_conf conf 
             on map.key_cluster_conf = conf.id_cluster_conf 
             and conf.conf_name = (
	         select conf_name from plproxy.db_conf where cluster_name = lower($1))
 order by conf.conf_map, map.hash_nr, map.key_partition;
$$
  LANGUAGE 'sql' VOLATILE SECURITY DEFINER;
