DROP TYPE plproxy.ret_get_cluster_partitions CASCADE;

CREATE TYPE plproxy.ret_get_cluster_partitions AS (
  id_partition int,
  connect_string text);

CREATE OR REPLACE FUNCTION plproxy.get_cluster_partitions(
    i_cluster_name text) 
RETURNS SETOF plproxy.ret_get_cluster_partitions AS $$
-- used by:: proxy pool to initialize internal maps
--   needed when run on all command is used in language 
--   and when partition number is found out from hash_map
-- br:: only partitons from active configuration are returned
--   db_proxy -- db_proxy_connection -- db_partition -- db_cluster
-- br:: proxy status must be either LOCKED, OFFLINE or ONLINE
-- if proxy is LOCKED ten exception is raised and script is stopped
--   these must be handled before script is run again
-- returns:: set of records
--    id_partition int - partition number 
--    connect_string text - connect string for that partition
-- example::
--   select * from plproxy.get_cluster_partitions('eventdb');
select p.id_partition, connect_string
from plproxy.db_partition p 
where key_cluster in (
    select key_cluster from plproxy.db_cluster_conf 
    where conf_name = (
        select conf_name from plproxy.db_conf 
	where cluster_name = lower($1)))
$$
  LANGUAGE 'sql' VOLATILE SECURITY DEFINER;


