DROP TYPE plproxy.ret_get_proxy_connections CASCADE;

CREATE TYPE plproxy.ret_get_proxy_connections AS (
  id_partition int4 ,
  conn_count_min int2 ,
  conn_count_max int2 ,
  connect_string text ,  
  partition_name text
);

CREATE OR REPLACE FUNCTION plproxy.get_proxy_connections(
    i_proxy_name text)
RETURNS SETOF plproxy.ret_get_proxy_connections AS $$
-- used by:: proxy pool to initialize internal maps
--   needed when pool is started or when configuration is changed or reloaded
-- br:: connections only for one proxy are returned
-- returns:: set of records
--   id_partition int4
--   conn_count_min int2
--   conn_count_max int2
--   connect_string text
-- example::
--   select * from plproxy.get_proxy_connections('central');  
    select part.id_partition, conn.conn_count_min, conn.conn_count_max, part.connect_string, partition_name
    from plproxy.db_proxy prox 
        inner join plproxy.db_proxy_connection conn on prox.id_proxy = conn.key_proxy
        inner join plproxy.db_partition part on conn.key_partition = part.id_partition
    where prox.proxy_name = $1;
$$ LANGUAGE 'sql' VOLATILE SECURITY DEFINER;
