CREATE OR REPLACE FUNCTION plproxy.set_master_active_conf(
    i_cluster_name text, 
    i_conf_name text) 
    RETURNS text AS $$
-- used by:: configuration change script to update configuration status in master database
--   in the end we use same function to change state but using different funtion gives 
--   possigility to do more on master side if logging or someting else is needed
-- br:: check that configuration with given keys exists
-- returns:: 200 Ok -- when successful
-- exceptions:: 400 Configuration (%) not found in cluster (%), Cannot activate
-- example::
--   select * from plproxy.set_master_activate_conf('userdb', 'userdb_v2');  
BEGIN	
    -- are names correct just to be safe
    if not exists (
       select 1 from plproxy.bp_cluster_conf 
       where conf_name = lower(i_conf_name)
       and key_cluster = (
           select id_cluster from plproxy.db_cluster where cluster_name = lower(i_cluster_name)))
    then
        raise exception '400 Configuration (%) not found in cluster (%), Cannot activate', 
            i_conf_name, i_cluster_name;
    end if;
    
    PERFORM plproxy.set_active_conf(i_cluster_name, i_conf_name);
    
    return '200 Ok';
END;
$$ LANGUAGE 'plpgsql' VOLATILE SECURITY DEFINER;
