DROP TYPE plproxy.ret_view_cluster_proxies CASCADE;

CREATE TYPE plproxy.ret_view_cluster_proxies AS (
  proxy_name text ,
  proxy_status text ,
  connect_string text   
);

CREATE OR REPLACE FUNCTION plproxy.view_cluster_proxies(
    i_cluster_name text)
  RETURNS SETOF plproxy.ret_view_cluster_proxies AS $$
-- used by:: configuration change scipt to get list of proxy servers
--   that are affected by configuration change ingiven cluster
--   list is used to connect to all these proxies and lock them
--   for configuration change
-- br:: it must contain all the proxies that service given cluster
--   db_proxy -- db_proxy_connection -- db_partition -- db_cluster
-- br:: proxy status must be either LOCKED, ONLINE
-- if proxy is LOCKED then calling script should not proceed
-- returns:: set of records
--   proxy_name text - used to alter proxy state later in script
--   proxy_status text - current status of the proxy 
--   connect_string text - used to connect to proxy
-- example::
--   select * from plproxy.get_cluster_proxies('userdb');  
  select proxy_name, null::text, null::text
  from plproxy.db_proxy prox 
  where id_proxy in (
    select key_proxy from plproxy.db_proxy_connection
    where key_partition in (
      select key_partition from plproxy.db_partition 
      where key_cluster = (
        select id_cluster from plproxy.db_cluster where cluster_name = lower($1))));
$$ LANGUAGE 'sql' VOLATILE SECURITY DEFINER;
