DROP TYPE plproxy.ret_view_partiton_changes CASCADE;

CREATE TYPE plproxy.ret_view_partiton_changes AS (
    source_name text ,
    source_connect text ,
    target_name text ,
    target_connect text );

CREATE OR REPLACE FUNCTION plproxy.view_partiton_changes(
    i_cluster_name text , -- name of the cluster modified (it's active conf is used as source)
    i_target_conf_name text ) -- name of the configuration that will be activated
RETURNS SETOF plproxy.ret_view_partiton_changes AS $$
-- used by:: configuration change script to get list of partitions
--   whose replication status must be checked
-- br:: we get all needed partitons from db_conf_partition where hash_nr
--   im new configuration are in different partition than in old one
--   this implies also that old partition must be replicated to new one
-- br:: currently active configuration is used as source configuration
-- returns:: set of records
--   source_name text -- replication master used for meaningful messages
--   source_connect text -- used to check if replication is ok 
--      and to end it when configuration gets updated
--   target_name text -- replication slave used for meaningful messages
--   target_connect text -- used to check if replication is 
-- example::
--   select * from plproxy.get_cluster_proxies('userdb');  
select src.partition_name as source_name
, src.connect_string as source_connect
, tgt.partition_name as target_name
, tgt.connect_string as target_connect
from 
( select distinct 
  source.key_partition as source_partition
  , target.key_partition as target_partition
  from
  ( select sp.key_partition, sp.hash_nr 
    from plproxy.db_conf_partition sp
    where sp.key_cluster_conf in (
      select id_cluster_conf from plproxy.db_cluster_conf 
      where conf_name = (
          select conf_name from plproxy.db_conf
	  where cluster_name = lower($1)))
  ) source
  left outer join
  ( select tp.key_partition, tp.hash_nr 
    from plproxy.db_conf_partition tp
    where tp.key_cluster_conf in (
      select id_cluster_conf from plproxy.db_cluster_conf where conf_name = lower($2))
  ) target
  on source.hash_nr = target.hash_nr
) chg
left outer join plproxy.db_partition src on chg.source_partition = src.id_partition
left outer join plproxy.db_partition tgt on chg.target_partition = tgt.id_partition
$$
  LANGUAGE 'sql' VOLATILE SECURITY DEFINER;
